############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 @tmshn <tmshn@r.recruit.co.jp>                                #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 MarcoFalke <falke.marco@gmail.com>                            #
# Copyright 2018 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 MarcoFalke <falke.marco@gmail.com>                            #
# Copyright 2019 Mark Browning <mark@cerebras.net>                             #
# Copyright 2019 Pavan Kunisetty <nagapavan@users.noreply.github.com>          #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Tim Gates <tim.gates@iress.com>                               #
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Florent Clarret <florent.clarret@gmail.com>                   #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Heitor Polidoro <14806300+heitorpolidoro@users.noreply.github.com>#
# Copyright 2023 Heitor Polidoro <heitor.polidoro@gmail.com>                   #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2023 vanya20074 <vanya20074@gmail.com>                             #
# Copyright 2024 Austin Sasko <austintyler0239@yahoo.com>                      #
# Copyright 2024 Den Stroebel <stroebs@users.noreply.github.com>               #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Bruno Didot <bdidot@gmail.com>                                #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Jakub Smolar <jakub.smolar@scylladb.com>                      #
# Copyright 2025 Michael Kukarkin <kukarkinmm@gmail.com>                       #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

import pytest

import github

from . import Framework


class PullRequest(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.repo = self.g.get_repo("PyGithub/PyGithub")
        self.pull = self.repo.get_pull(31)

        marco_repo = self.g.get_repo("MarcoFalke/PyGithub", lazy=True)
        self.pullIssue256Closed = marco_repo.get_pull(1).complete()
        self.pullIssue256Merged = marco_repo.get_pull(2).complete()
        self.pullIssue256Conflict = marco_repo.get_pull(3).complete()
        self.pullIssue256Uncached = marco_repo.get_pull(4).complete()

        flo_repo = self.g.get_repo("FlorentClarret/PyGithub")
        self.pullMaintainerCanModify = flo_repo.get_pull(2)

        self.delete_restore_repo = self.g.get_repo("austinsasko/PyGithub")
        self.delete_restore_pull = self.delete_restore_repo.get_pull(21)

    def testAttributesIssue256(self):
        self.assertEqual(
            self.pullIssue256Closed.closed_at,
            datetime(2018, 5, 22, 14, 50, 43, tzinfo=timezone.utc),
        )
        self.assertEqual(
            self.pullIssue256Merged.closed_at,
            datetime(2018, 5, 22, 14, 53, 13, tzinfo=timezone.utc),
        )
        self.assertEqual(self.pullIssue256Conflict.closed_at, None)
        self.assertEqual(self.pullIssue256Uncached.closed_at, None)

        self.assertEqual(self.pullIssue256Closed.state, "closed")
        self.assertEqual(self.pullIssue256Merged.state, "closed")
        self.assertEqual(self.pullIssue256Conflict.state, "open")
        self.assertEqual(self.pullIssue256Uncached.state, "open")

        self.assertFalse(self.pullIssue256Closed.merged)
        self.assertTrue(self.pullIssue256Merged.merged)
        self.assertFalse(self.pullIssue256Conflict.merged)
        self.assertFalse(self.pullIssue256Uncached.merged)

        self.assertEqual(self.pullIssue256Closed.mergeable, None)
        self.assertEqual(self.pullIssue256Merged.mergeable, None)
        self.assertTrue(self.pullIssue256Conflict.mergeable)
        self.assertEqual(self.pullIssue256Uncached.mergeable, None)

        self.assertEqual(self.pullIssue256Closed.mergeable_state, "unknown")
        self.assertEqual(self.pullIssue256Merged.mergeable_state, "unknown")
        self.assertEqual(self.pullIssue256Conflict.mergeable_state, "clean")
        self.assertEqual(self.pullIssue256Uncached.mergeable_state, "unknown")

    def testAttributes(self):
        self.assertEqual(
            self.pull._links,
            {
                "self": {"href": "https://api.github.com/repos/PyGithub/PyGithub/pulls/31"},
                "html": {"href": "https://github.com/PyGithub/PyGithub/pull/31"},
                "issue": {"href": "https://api.github.com/repos/PyGithub/PyGithub/issues/31"},
                "comments": {"href": "https://api.github.com/repos/PyGithub/PyGithub/issues/31/comments"},
                "review_comments": {"href": "https://api.github.com/repos/PyGithub/PyGithub/pulls/31/comments"},
                "review_comment": {"href": "https://api.github.com/repos/PyGithub/PyGithub/pulls/comments{/number}"},
                "commits": {"href": "https://api.github.com/repos/PyGithub/PyGithub/pulls/31/commits"},
                "statuses": {
                    "href": "https://api.github.com/repos/PyGithub/PyGithub/statuses/8a4f306d4b223682dd19410d4a9150636ebe4206"
                },
            },
        )
        self.assertIsNone(self.pull.active_lock_reason)
        self.assertEqual(self.pull.additions, 511)
        self.assertEqual(self.pull.assignee.login, "jacquev6")
        self.assertListKeyEqual(self.pull.assignees, lambda a: a.login, ["jacquev6"])
        self.assertEqual(self.pull.author_association, "MEMBER")
        self.assertIsNone(self.pull.auto_merge)
        self.assertEqual(self.pull.base.label, "PyGithub:topic/RewriteWithGeneratedCode")
        self.assertEqual(self.pull.base.sha, "ed866fc43833802ab553e5ff8581c81bb00dd433")
        self.assertEqual(self.pull.base.user.login, "PyGithub")
        self.assertEqual(self.pull.base.ref, "topic/RewriteWithGeneratedCode")
        self.assertEqual(self.pull.base.repo.full_name, "PyGithub/PyGithub")
        self.assertEqual(self.pull.body, "Body edited by PyGithub\n")
        self.assertEqual(self.pull.changed_files, 45)
        self.assertEqual(
            self.pull.closed_at,
            datetime(2012, 5, 27, 10, 29, 7, tzinfo=timezone.utc),
        )
        self.assertEqual(self.pull.comments, 1)
        self.assertEqual(self.pull.comments_url, "https://api.github.com/repos/PyGithub/PyGithub/issues/31/comments")
        self.assertEqual(self.pull.commits, 3)
        self.assertEqual(self.pull.commits_url, "https://api.github.com/repos/PyGithub/PyGithub/pulls/31/commits")
        self.assertEqual(
            self.pull.created_at,
            datetime(2012, 5, 27, 9, 25, 36, tzinfo=timezone.utc),
        )
        self.assertEqual(self.pull.deletions, 384)
        self.assertEqual(self.pull.diff_url, "https://github.com/PyGithub/PyGithub/pull/31.diff")
        self.assertEqual(self.pull.draft, False)
        self.assertEqual(self.pull.head.ref, "master")
        self.assertEqual(self.pull.html_url, "https://github.com/PyGithub/PyGithub/pull/31")
        self.assertEqual(self.pull.id, 1436215)
        self.assertEqual(
            self.pull.issue_url,
            "https://api.github.com/repos/PyGithub/PyGithub/issues/31",
        )
        self.assertListKeyEqual(self.pull.labels, lambda a: a.name, [])
        self.assertEqual(self.pull.locked, False)
        self.assertEqual(self.pull.maintainer_can_modify, False)
        self.assertEqual(self.pull.merge_commit_sha, "28ae6dd10ebccd5eaf8db8dacb5b699ee7f4a663")
        self.assertFalse(self.pull.mergeable)
        self.assertEqual(self.pull.mergeable_state, "dirty")
        self.assertEqual(self.pull.merged, True)
        self.assertEqual(self.pull.merged_at, datetime(2012, 5, 27, 10, 29, 7, tzinfo=timezone.utc))
        self.assertEqual(self.pull.merged_by.login, "jacquev6")
        self.assertEqual(self.pull.milestone.number, 1)
        self.assertEqual(self.pull.node_id, "MDExOlB1bGxSZXF1ZXN0MTQzNjIxNQ==")
        self.assertEqual(self.pull.number, 31)
        self.assertEqual(self.pull.patch_url, "https://github.com/PyGithub/PyGithub/pull/31.patch")
        self.assertFalse(self.pull.rebaseable)
        self.assertTrue(self.pull.merged)
        self.assertEqual(
            self.pull.merged_at,
            datetime(2012, 5, 27, 10, 29, 7, tzinfo=timezone.utc),
        )
        self.assertEqual(self.pull.merged_by.login, "jacquev6")
        self.assertEqual(self.pull.number, 31)
        self.assertEqual(self.pull.patch_url, "https://github.com/PyGithub/PyGithub/pull/31.patch")
        self.assertEqual(self.pull.requested_reviewers[0].login, "sfdye")
        self.assertEqual(self.pull.requested_teams[0].id, 123)
        self.assertEqual(
            self.pull.review_comment_url, "https://api.github.com/repos/PyGithub/PyGithub/pulls/comments{/number}"
        )
        self.assertEqual(self.pull.review_comments, 2)
        self.assertEqual(
            self.pull.review_comments_url, "https://api.github.com/repos/PyGithub/PyGithub/pulls/31/comments"
        )
        self.assertEqual(self.pull.state, "closed")
        self.assertEqual(
            self.pull.statuses_url,
            "https://api.github.com/repos/PyGithub/PyGithub/statuses/8a4f306d4b223682dd19410d4a9150636ebe4206",
        )
        self.assertEqual(self.pull.title, "Title edited by PyGithub")
        self.assertEqual(
            self.pull.updated_at,
            datetime(2018, 6, 25, 12, 54, 43, tzinfo=timezone.utc),
        )
        self.assertEqual(self.pull.url, "https://api.github.com/repos/PyGithub/PyGithub/pulls/31")
        self.assertEqual(self.pull.user.login, "jacquev6")
        self.assertEqual(self.pull.draft, False)
        self.assertEqual(self.pull.maintainer_can_modify, False)
        self.assertEqual(
            repr(self.pull),
            'PullRequest(title="Title edited by PyGithub", number=31)',
        )
        self.assertEqual(
            repr(self.pull.base),
            'PullRequestPart(sha="ed866fc43833802ab553e5ff8581c81bb00dd433")',
        )
        self.assertTrue(self.pullIssue256Conflict.rebaseable)

    def testLazyAttributes(self):
        pull = self.g.withLazy(True).get_repo("lazy/repo").get_pull(42)
        self.assertEqual(str(pull), "PullRequest(title=None, number=42)")
        self.assertEqual(pull.number, 42)
        self.assertEqual(pull.url, "/repos/lazy/repo/pulls/42")

    def testCreateComment(self):
        commit = self.repo.get_commit("8a4f306d4b223682dd19410d4a9150636ebe4206")
        comment = self.pull.create_comment("Comment created by PyGithub", commit, "src/github/Issue.py", 5)
        self.assertEqual(comment.id, 886298)

    def testCreateReviewCommentInReplyTo(self):
        commit = self.repo.get_commit("8a4f306d4b223682dd19410d4a9150636ebe4206")
        comment = self.pull.create_review_comment(
            "Comment created by PyGithub",
            commit,
            "src/github/Issue.py",
            5,
            in_reply_to=42,
        )
        self.assertEqual(comment.id, 886298)

    def testCreateReviewCommentSubjectType(self):
        commit = self.repo.get_commit("8a4f306d4b223682dd19410d4a9150636ebe4206")
        comment = self.pull.create_review_comment(
            "Comment created by PyGithub",
            commit,
            "src/github/Issue.py",
            5,
            subject_type="file",
        )
        self.assertEqual(comment.id, 886298)

    def testCreateMultilineReviewComment(self):
        commit = self.repo.get_commit("8a4f306d4b223682dd19410d4a9150636ebe4206")
        comment = self.pull.create_review_comment(
            "Comment created by PyGithub",
            commit,
            "src/github/Issue.py",
            10,
            start_line=5,
        )
        self.assertEqual(comment.id, 886298)

    def testCreateMultilineReviewCommentAsSuggestion(self):
        commit = self.repo.get_commit("8a4f306d4b223682dd19410d4a9150636ebe4206")
        comment = self.pull.create_review_comment(
            "Comment created by PyGithub",
            commit,
            "src/github/Issue.py",
            10,
            start_line=5,
            as_suggestion=True,
        )
        self.assertEqual(comment.id, 886298)
        self.assertEqual(comment.body, "```suggestion\nComment created by PyGithub\n```")

    def testCreateMultilineReviewCommentChoosingSide(self):
        commit = self.repo.get_commit("8a4f306d4b223682dd19410d4a9150636ebe4206")
        comment = self.pull.create_review_comment(
            "Comment created by PyGithub",
            commit,
            "src/github/Issue.py",
            10,
            start_line=5,
            side="RIGHT",
            start_side="RIGHT",
        )
        self.assertEqual(comment.id, 886298)

    def testGetComments(self):
        epoch = datetime(1970, 1, 1, 0, 0)
        comments = self.pull.get_comments(sort="updated", direction="desc", since=epoch)
        self.assertListKeyEqual(comments, lambda c: c.id, [197784357, 1580134])

    def testCreateIssueComment(self):
        comment = self.pull.create_issue_comment("Issue comment created by PyGithub")
        self.assertEqual(comment.id, 8387331)

    def testGetIssueComments(self):
        self.assertListKeyEqual(self.pull.get_issue_comments(), lambda c: c.id, [8387331])

    def testGetIssueComment(self):
        comment = self.pull.get_issue_comment(8387331)
        self.assertEqual(comment.body, "Issue comment created by PyGithub")

    def testGetIssueEvents(self):
        self.assertListKeyEqual(
            self.pull.get_issue_events(),
            lambda e: e.id,
            [16349963, 16350729, 16350730, 16350731, 28469043, 98136335],
        )

    def testGetIssueTimeline(self):
        self.assertListKeyEqual(
            self.pull.get_issue_timeline(),
            lambda t: t.event,
            [
                "committed",
                "committed",
                "committed",
                "subscribed",
                "referenced",
                "merged",
                "closed",
                "commented",
                "line-commented",
                "assigned",
                "labeled",
                "cross-referenced",
                "reviewed",
                "reviewed",
            ],
        )

    def testGetReviewComments(self):
        epoch = datetime(1970, 1, 1, 0, 0)
        comments = self.pull.get_review_comments(sort="updated", direction="desc", since=epoch)
        self.assertListKeyEqual(comments, lambda c: c.id, [197784357, 1580134])
        self.assertListKeyEqual(comments, lambda c: c.pull_request_review_id, [131593233, None])

    def testReviewRequests(self):
        self.pull.create_review_request(reviewers="sfdye", team_reviewers="pygithub-owners")
        review_requests = self.pull.get_review_requests()
        self.assertListKeyEqual(review_requests[0], lambda c: c.login, ["sfdye"])
        self.assertListKeyEqual(review_requests[1], lambda c: c.slug, ["pygithub-owners"])
        self.pull.delete_review_request(reviewers="sfdye")
        review_requests = self.pull.get_review_requests()
        self.assertEqual(list(review_requests[0]), [])
        self.assertListKeyEqual(review_requests[1], lambda c: c.slug, ["pygithub-owners"])

    def testEditWithoutArguments(self):
        self.pull.edit()

    def testEditWithAllArguments(self):
        self.pullMaintainerCanModify.edit(
            "Title edited by PyGithub",
            "Body edited by PyGithub",
            "open",
            "master",
            True,
        )
        self.assertEqual(self.pullMaintainerCanModify.title, "Title edited by PyGithub")
        self.assertEqual(self.pullMaintainerCanModify.body, "Body edited by PyGithub")
        self.assertEqual(self.pullMaintainerCanModify.state, "open")
        self.assertEqual(self.pullMaintainerCanModify.base.ref, "master")
        self.assertTrue(self.pullMaintainerCanModify.maintainer_can_modify)

    def testGetCommits(self):
        self.assertListKeyEqual(
            self.pull.get_commits(),
            lambda c: c.sha,
            [
                "4aadfff21cdd2d2566b0e4bd7309c233b5f4ae23",
                "93dcae5cf207de376c91d0599226e7c7563e1d16",
                "8a4f306d4b223682dd19410d4a9150636ebe4206",
            ],
        )

    def testGetFiles(self):
        self.assertListKeyEqual(
            self.pull.get_files(),
            lambda f: f.filename,
            [
                "codegen/templates/GithubObject.py",
                "src/github/AuthenticatedUser.py",
                "src/github/Authorization.py",
                "src/github/Branch.py",
                "src/github/Commit.py",
                "src/github/CommitComment.py",
                "src/github/CommitFile.py",
                "src/github/CommitStats.py",
                "src/github/Download.py",
                "src/github/Event.py",
                "src/github/Gist.py",
                "src/github/GistComment.py",
                "src/github/GistHistoryState.py",
                "src/github/GitAuthor.py",
                "src/github/GitBlob.py",
                "src/github/GitCommit.py",
                "src/github/GitObject.py",
                "src/github/GitRef.py",
                "src/github/GitTag.py",
                "src/github/GitTree.py",
                "src/github/GitTreeElement.py",
                "src/github/Hook.py",
                "src/github/Issue.py",
                "src/github/IssueComment.py",
                "src/github/IssueEvent.py",
                "src/github/Label.py",
                "src/github/Milestone.py",
                "src/github/NamedUser.py",
                "src/github/Organization.py",
                "src/github/Permissions.py",
                "src/github/Plan.py",
                "src/github/PullRequest.py",
                "src/github/PullRequestComment.py",
                "src/github/PullRequestFile.py",
                "src/github/Repository.py",
                "src/github/RepositoryKey.py",
                "src/github/Tag.py",
                "src/github/Team.py",
                "src/github/UserKey.py",
                "test/Issue.py",
                "test/IssueEvent.py",
                "test/ReplayData/Issue.testAddAndRemoveLabels.txt",
                "test/ReplayData/Issue.testDeleteAndSetLabels.txt",
                "test/ReplayData/Issue.testGetLabels.txt",
                "test/ReplayData/IssueEvent.setUp.txt",
            ],
        )

    def testGetLabels(self):
        self.assertListKeyEqual(self.pull.get_labels(), lambda lb: lb.name, ["wip", "refactoring"])

    def testAddAndRemoveLabels(self):
        wip = self.repo.get_label("wip")
        refactoring = self.repo.get_label("refactoring")
        self.assertListKeyEqual(
            self.pull.get_labels(),
            lambda lb: lb.name,
            ["wip", "refactoring", "improvement"],
        )
        self.pull.remove_from_labels(wip)
        self.assertListKeyEqual(self.pull.get_labels(), lambda lb: lb.name, ["refactoring", "improvement"])
        self.pull.remove_from_labels(refactoring)
        self.assertListKeyEqual(self.pull.get_labels(), lambda lb: lb.name, ["improvement"])
        self.pull.add_to_labels(wip, refactoring)
        self.assertListKeyEqual(
            self.pull.get_labels(),
            lambda lb: lb.name,
            ["wip", "refactoring", "improvement"],
        )

    def testAddAndRemoveLabelsWithStringArguments(self):
        wip = "wip"
        refactoring = "refactoring"
        self.assertListKeyEqual(
            self.pull.get_labels(),
            lambda lb: lb.name,
            ["wip", "refactoring", "improvement"],
        )
        self.pull.remove_from_labels(wip)
        self.assertListKeyEqual(self.pull.get_labels(), lambda lb: lb.name, ["refactoring", "improvement"])
        self.pull.remove_from_labels(refactoring)
        self.assertListKeyEqual(self.pull.get_labels(), lambda lb: lb.name, ["improvement"])
        self.pull.add_to_labels(wip, refactoring)
        self.assertListKeyEqual(
            self.pull.get_labels(),
            lambda lb: lb.name,
            ["wip", "refactoring", "improvement"],
        )

    def testDeleteAndSetLabels(self):
        wip = self.repo.get_label("wip")
        refactoring = self.repo.get_label("refactoring")
        self.assertListKeyEqual(
            self.pull.get_labels(),
            lambda lb: lb.name,
            ["wip", "refactoring", "improvement"],
        )
        self.pull.delete_labels()
        self.assertListKeyEqual(self.pull.get_labels(), None, [])
        self.pull.set_labels(wip, refactoring)
        self.assertListKeyEqual(self.pull.get_labels(), lambda lb: lb.name, ["wip", "refactoring"])

    def testDeleteAndSetLabelsWithStringArguments(self):
        wip = "wip"
        refactoring = "refactoring"
        self.assertListKeyEqual(
            self.pull.get_labels(),
            lambda lb: lb.name,
            ["wip", "refactoring", "improvement"],
        )
        self.pull.delete_labels()
        self.assertListKeyEqual(self.pull.get_labels(), None, [])
        self.pull.set_labels(wip, refactoring)
        self.assertListKeyEqual(self.pull.get_labels(), lambda lb: lb.name, ["wip", "refactoring"])

    def testMerge(self):
        self.assertFalse(self.pull.is_merged())
        status = self.pull.merge()
        self.assertEqual(status.sha, "688208b1a5a074871d0e9376119556897439697d")
        self.assertTrue(status.merged)
        self.assertEqual(status.message, "Pull Request successfully merged")
        self.assertTrue(self.pull.is_merged())
        self.assertEqual(
            repr(status),
            'PullRequestMergeStatus(sha="688208b1a5a074871d0e9376119556897439697d", merged=True)',
        )

    def testMergeWithCommitMessage(self):
        self.repo.get_pull(39).merge("Custom commit message created by PyGithub")

    def testAddAndRemoveAssignees(self):
        user1 = "jayfk"
        user2 = self.g.get_user("jzelinskie")
        self.assertListKeyEqual(self.pull.assignees, lambda a: a.login, ["jacquev6"])
        url = self.pull.url
        self.pull.add_to_assignees(user1, user2)
        self.assertListKeyEqual(
            self.pull.assignees,
            lambda a: a.login,
            ["jacquev6", "stuglaser", "jayfk", "jzelinskie"],
        )
        self.assertEqual(self.pull.url, url)
        self.pull.remove_from_assignees(user1, user2)
        self.assertListKeyEqual(self.pull.assignees, lambda a: a.login, ["jacquev6", "stuglaser"])
        self.assertEqual(self.pull.url, url)

    def testUpdateBranch(self):
        self.assertTrue(self.pull.update_branch("addaebea821105cf6600441f05ff2b413ab21a36"))
        self.assertTrue(self.pull.update_branch())

    def testConvertToDraft(self):
        ready_pr = self.g.get_repo("didot/PyGithub", lazy=True).get_pull(1)
        self.assertFalse(ready_pr.draft)
        response = ready_pr.convert_to_draft()
        self.assertTrue(ready_pr.draft)
        assert response == {
            "clientMutationId": None,
            "pullRequest": {
                "isDraft": True,
            },
        }

    def testMarkReadyForReview(self):
        draft_pr = self.g.get_repo("didot/PyGithub", lazy=True).get_pull(2)
        self.assertTrue(draft_pr.draft)
        response = draft_pr.mark_ready_for_review()
        self.assertFalse(draft_pr.draft)
        assert response == {
            "clientMutationId": None,
            "pullRequest": {
                "isDraft": False,
            },
        }

    def testDeleteOnMerge(self):
        self.assertTrue(self.delete_restore_repo.get_branch(self.delete_restore_pull.head.ref))
        self.assertFalse(self.delete_restore_pull.is_merged())
        status = self.delete_restore_pull.merge(delete_branch=True)
        self.assertTrue(status.merged)
        self.assertTrue(self.delete_restore_pull.is_merged())
        with self.assertRaises(github.UnknownObjectException) as raisedexp:
            self.delete_restore_repo.get_branch(self.delete_restore_pull.head.ref)
        self.assertEqual(raisedexp.exception.message, "Branch not found")
        self.assertEqual(
            raisedexp.exception.data,
            {
                "documentation_url": "https://docs.github.com/rest/reference/repos#get-a-branch",
                "message": "Branch not found",
            },
        )

    def testRestoreBranch(self):
        with self.assertRaises(github.UnknownObjectException) as raisedexp:
            self.delete_restore_repo.get_branch(self.delete_restore_pull.head.ref)
        self.assertEqual(raisedexp.exception.message, "Branch not found")
        self.assertEqual(raisedexp.exception.status, 404)
        self.assertEqual(
            raisedexp.exception.data,
            {
                "documentation_url": "https://docs.github.com/rest/reference/repos#get-a-branch",
                "message": "Branch not found",
            },
        )
        self.assertTrue(self.delete_restore_pull.restore_branch())
        self.assertTrue(self.delete_restore_repo.get_branch(self.delete_restore_pull.head.ref))

    def testDeleteBranch(self):
        self.assertTrue(self.delete_restore_repo.get_branch(self.delete_restore_pull.head.ref))
        self.delete_restore_pull.delete_branch(force=False)
        with self.assertRaises(github.UnknownObjectException) as raisedexp:
            self.delete_restore_repo.get_branch(self.delete_restore_pull.head.ref)
        self.assertEqual(raisedexp.exception.message, "Branch not found")
        self.assertEqual(raisedexp.exception.status, 404)
        self.assertEqual(
            raisedexp.exception.data,
            {
                "documentation_url": "https://docs.github.com/rest/reference/repos#get-a-branch",
                "message": "Branch not found",
            },
        )

    def testForceDeleteBranch(self):
        self.assertTrue(self.delete_restore_repo.get_branch(self.delete_restore_pull.head.ref))
        self.assertEqual(self.delete_restore_pull.delete_branch(force=True), None)
        with self.assertRaises(github.UnknownObjectException) as raisedexp:
            self.delete_restore_repo.get_branch(self.delete_restore_pull.head.ref)
        self.assertEqual(raisedexp.exception.message, "Branch not found")
        self.assertEqual(raisedexp.exception.status, 404)
        self.assertEqual(
            raisedexp.exception.data,
            {
                "documentation_url": "https://docs.github.com/rest/reference/repos#get-a-branch",
                "message": "Branch not found",
            },
        )

    def testEnableAutomerge(self):
        # To reproduce this, the PR repository need to have the "Allow auto-merge" option enabled
        response = self.pull.enable_automerge(
            merge_method="SQUASH",
            author_email="foo@example.com",
            client_mutation_id="1234",
            commit_body="body of the commit",
            commit_headline="The commit headline",
            expected_head_oid="0283d46537193f1fed7d46859f15c5304b9836f9",
        )
        assert response == {
            "actor": {
                "avatarUrl": "https://avatars.githubusercontent.com/u/14806300?u=786f9f8ef8782d45381b01580f7f7783cf9c7e37&v=4",
                "login": "heitorpolidoro",
                "resourcePath": "/heitorpolidoro",
                "url": "https://github.com/heitorpolidoro",
            },
            "clientMutationId": None,
        }

    def testEnableAutomergeDefaultValues(self):
        # To reproduce this, the PR repository need to have the "Allow auto-merge" option enabled
        # The default values are:
        # - merge_method = "MERGE"
        self.pull.enable_automerge()

    def testEnableAutomergeNotValidMergeMethod(self):
        with pytest.raises(AssertionError):
            self.pull.enable_automerge(merge_method="INVALID")

    def testEnableAutomergeError(self):
        # To reproduce this, the PR repository need to have the "Allow auto-merge" option disabled
        with pytest.raises(github.GithubException) as error:
            self.pull.enable_automerge()
        assert error.value.message is None
        assert error.value.status == 400
        assert error.value.data == {
            "data": {"enablePullRequestAutoMerge": None},
            "errors": [
                {
                    "locations": [{"column": 81, "line": 1}],
                    "message": "Pull request Auto merge is not allowed for this repository",
                    "path": ["enablePullRequestAutoMerge"],
                    "type": "UNPROCESSABLE",
                }
            ],
        }

    def testDisableAutomerge(self):
        response = self.pull.disable_automerge()
        assert response == {
            "actor": {
                "avatarUrl": "https://avatars.githubusercontent.com/u/14806300?u=786f9f8ef8782d45381b01580f7f7783cf9c7e37&v=4",
                "login": "heitorpolidoro",
                "resourcePath": "/heitorpolidoro",
                "url": "https://github.com/heitorpolidoro",
            },
            "clientMutationId": None,
        }



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 Hayden Fuss <wifu1234@gmail.com>                              #
# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from . import Framework


class SourceImport(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.user = self.g.get_user("brix4dayz")
        self.repo = self.user.get_repo("source-import-test")
        self.source_import = self.repo.get_source_import()

    def testAttributes(self):
        self.assertEqual(self.source_import.authors_count, 1)
        self.assertEqual(
            self.source_import.authors_url,
            "https://api.github.com/repos/brix4dayz/source-import-test/import/authors",
        )
        self.assertIsNone(self.source_import.commit_count)
        self.assertIsNone(self.source_import.error_message)
        self.assertIsNone(self.source_import.failed_step)
        self.assertEqual(self.source_import.has_large_files, False)
        self.assertEqual(
            self.source_import.html_url,
            "https://github.com/brix4dayz/source-import-test/import",
        )
        self.assertIsNone(self.source_import.import_percent)
        self.assertEqual(self.source_import.large_files_count, 0)
        self.assertEqual(self.source_import.large_files_size, 0)
        self.assertIsNone(self.source_import.message)
        self.assertIsNone(self.source_import.project_choices)
        self.assertIsNone(self.source_import.push_percent)
        self.assertEqual(
            self.source_import.repository_url,
            "https://api.github.com/repos/brix4dayz/source-import-test",
        )
        self.assertEqual(self.source_import.status, "complete")
        self.assertEqual(self.source_import.status_text, "Done")
        self.assertIsNone(self.source_import.svc_root)
        self.assertIsNone(self.source_import.svn_root)
        self.assertIsNone(self.source_import.tfvc_project)
        self.assertEqual(
            self.source_import.url,
            "https://api.github.com/repos/brix4dayz/source-import-test/import",
        )
        self.assertEqual(self.source_import.use_lfs, "undecided")
        self.assertEqual(self.source_import.vcs, "mercurial")
        self.assertEqual(self.source_import.vcs_url, "https://bitbucket.org/hfuss/source-import-test")

        self.assertEqual(
            self.source_import.__repr__(),
            'SourceImport(vcs_url="https://bitbucket.org/hfuss/source-import-test", url="https://api.github.com/repos/brix4dayz/source-import-test/import", status="complete", repository_url="https://api.github.com/repos/brix4dayz/source-import-test")',
        )

    def testUpdate(self):
        # The real test is that update() method passes the header
        update_ret = self.source_import.update()
        self.assertTrue(update_ret)
        self.assertEqual(self.source_import.status, "complete")



#!/usr/bin/env python
############################ Copyrights and license ############################
#                                                                              #
# Copyright 2022 Aleksei Fedotov <lexa@cfotr.com>                              #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Trim21 <trim21.me@gmail.com>                                  #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

import github

from . import Framework


class Artifact(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.repo = self.g.get_repo("transmission-web-control/transmission-web-control")

    def testGetArtifactsFromWorkflow(self):
        artifact = self.repo.get_workflow_run(5138169628).get_artifacts()[0]

        self.assertEqual(artifact.name, "build-tar")
        self.assertFalse(artifact.expired)
        self.assertEqual(repr(artifact), 'Artifact(name="build-tar", id=724958104)')

    def testGetArtifactsFromRepoWithName(self):
        artifacts = self.repo.get_artifacts(name="build-tar")
        self.assertEqual(artifacts.totalCount, 296)
        assert all(x.name == "build-tar" for x in artifacts)

        artifact = artifacts[0]

        self.assertEqual(artifact.name, "build-tar")
        self.assertEqual(repr(artifact), 'Artifact(name="build-tar", id=724959170)')

    def testGetSingleArtifactFromRepo(self):
        artifact = self.repo.get_artifact(719509139)

        self.assertEqual(artifact.name, "build-zip")
        self.assertFalse(artifact.expired)
        self.assertEqual(repr(artifact), 'Artifact(name="build-zip", id=719509139)')

    def testGetArtifactsFromRepo(self):
        artifact_id = 719509139
        artifacts = self.repo.get_artifacts()
        for item in artifacts:
            if item.id == artifact_id:
                artifact = item
                break
        else:
            assert False, f"No artifact {artifact_id} is found"

        self.assertEqual(
            repr(artifact),
            f'Artifact(name="build-zip", id={artifact_id})',
        )

    def testGetNonexistentArtifact(self):
        artifact_id = 396724437
        repo_name = "lexa/PyGithub"
        repo = self.g.get_repo(repo_name)
        with self.assertRaises(github.GithubException):
            repo.get_artifact(artifact_id)

    def testDelete(self):
        artifact_id = 396724439
        repo_name = "lexa/PyGithub"
        repo = self.g.get_repo(repo_name)
        artifact = repo.get_artifact(artifact_id)
        self.assertTrue(artifact.delete())
        with self.assertRaises(github.GithubException):
            repo.get_artifact(artifact_id)



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2022 Marco Köpcke <hello@parakoopa.de>                             #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2023 Oskar Jansson <56458534+janssonoskar@users.noreply.github.com>#
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from tests import Framework


class Autolink(Framework.TestCase):
    def setUp(self):
        super().setUp()
        # When recording test, be sure to create a autolink for yourself on
        # Github and update it here.
        links = [x for x in self.g.get_user("theCapypara").get_repo("PyGithub").get_autolinks() if x.id == 209614]
        self.assertEqual(1, len(links), "There must be exactly one autolink with the ID 209614.")
        self.link = links[0]

    def testAttributes(self):
        self.assertEqual(self.link.id, 209614)
        self.assertEqual(self.link.key_prefix, "DUMMY-")
        self.assertIsNone(self.link.updated_at)
        self.assertEqual(self.link.url_template, "https://github.com/PyGithub/PyGithub/issues/<num>")
        self.assertEqual(self.link.is_alphanumeric, True)



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2026 Matt Davis <35502728+matt-davis27@users.noreply.github.com>   #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

import github.CodeScanAlert
import github.PaginatedList

from . import Framework


class CodeScanAlert(Framework.TestCase):
    alert: github.CodeScanAlert.CodeScanAlert

    def setUp(self):
        super().setUp()
        self.repo = self.g.get_repo("matt-davis27/PyGithub")

    def testAttributes(self):
        alert = self.repo.get_codescan_alert(1)
        self.assertEqual(alert.number, 1)
        self.assertEqual(alert.created_at, datetime(2025, 8, 22, 23, 38, 23, tzinfo=timezone.utc))
        self.assertEqual(alert.updated_at, datetime(2025, 8, 25, 16, 3, 10, tzinfo=timezone.utc))
        self.assertEqual(alert.url, "https://api.github.com/repos/matt-davis27/PyGithub/code-scanning/alerts/1")
        self.assertEqual(alert.html_url, "https://github.com/matt-davis27/PyGithub/security/code-scanning/1")
        self.assertEqual(alert.state, "fixed")
        self.assertEqual(alert.fixed_at, datetime(2025, 8, 25, 16, 3, 9, tzinfo=timezone.utc))
        self.assertIsNone(alert.dismissed_by)
        self.assertIsNone(alert.dismissed_at)
        self.assertIsNone(alert.dismissed_reason)
        self.assertIsNone(alert.dismissed_comment)
        self.assertEqual(alert.rule.id, "actions/missing-workflow-permissions")
        self.assertEqual(alert.rule.severity, "warning")
        self.assertEqual(alert.rule.description, "Workflow does not contain permissions")
        self.assertEqual(alert.rule.name, "actions/missing-workflow-permissions")
        self.assertEqual(alert.rule.tags[0], "actions")
        self.assertEqual(alert.rule.tags[1], "external/cwe/cwe-275")
        self.assertEqual(alert.rule.tags[2], "maintainability")
        self.assertEqual(alert.rule.tags[3], "security")
        self.assertEqual(
            alert.rule.full_description,
            "Workflows should contain explicit permissions to restrict the scope of the default GITHUB_TOKEN.",
        )
        self.assertTrue(alert.rule.help.startswith("## Overview"))
        self.assertEqual(alert.rule.security_severity_level, "medium")
        self.assertEqual(alert.tool.name, "CodeQL")
        self.assertIsNone(alert.tool.guid)
        self.assertEqual(alert.tool.version, "2.22.4")
        self.assertEqual(alert.most_recent_instance.ref, "refs/heads/main")
        self.assertEqual(alert.most_recent_instance.analysis_key, ".github/workflows/codeql.yml:analyze")
        self.assertEqual(alert.most_recent_instance.environment, '{"build-mode":"none","language":"actions"}')
        self.assertEqual(alert.most_recent_instance.category, "/language:actions")
        self.assertEqual(alert.most_recent_instance.state, "fixed")
        self.assertEqual(alert.most_recent_instance.commit_sha, "908396804d41cdb1d0c0538b97f25a81383ee61b")
        self.assertEqual(
            alert.most_recent_instance.message["text"],
            "Actions job or workflow does not limit the permissions of the GITHUB_TOKEN. Consider setting an explicit permissions block, using the following as a minimal starting point: {{contents: read}}",
        )
        self.assertEqual(alert.most_recent_instance.location.path, ".github/workflows/lint.yml")
        self.assertEqual(alert.most_recent_instance.location.start_line, 12)
        self.assertEqual(alert.most_recent_instance.location.end_line, 29)
        self.assertEqual(alert.most_recent_instance.location.start_column, 5)
        self.assertEqual(alert.most_recent_instance.location.end_column, 3)
        self.assertEqual(len(alert.most_recent_instance.classifications), 0)
        self.assertEqual(
            alert.instances_url, "https://api.github.com/repos/matt-davis27/PyGithub/code-scanning/alerts/1/instances"
        )
        self.assertIsNone(alert.dismissal_approved_by)

    def testMultipleAlerts(self):
        multiple_alerts = self.repo.get_codescan_alerts()
        self.assertIsInstance(multiple_alerts, github.PaginatedList.PaginatedList)
        self.assertIsInstance(multiple_alerts[0], github.CodeScanAlert.CodeScanAlert)
        alert_list = [alert for alert in multiple_alerts]
        self.assertEqual(len(alert_list), 14)  # Update this when more alerts are added to the PyGithub repo

        test_alert = alert_list[
            -1
        ]  # Alerts are returned in descending order, so the first alert is the most recent and the last alert is the oldest
        # Everything below is the same as testAttributes. This is just to make sure the list works.
        self.assertEqual(test_alert.number, 1)
        self.assertEqual(test_alert.created_at, datetime(2025, 8, 22, 23, 38, 23, tzinfo=timezone.utc))
        self.assertEqual(test_alert.updated_at, datetime(2025, 8, 25, 16, 3, 10, tzinfo=timezone.utc))
        self.assertEqual(test_alert.url, "https://api.github.com/repos/matt-davis27/PyGithub/code-scanning/alerts/1")
        self.assertEqual(test_alert.html_url, "https://github.com/matt-davis27/PyGithub/security/code-scanning/1")
        self.assertEqual(test_alert.state, "fixed")
        self.assertEqual(test_alert.fixed_at, datetime(2025, 8, 25, 16, 3, 9, tzinfo=timezone.utc))
        self.assertIsNone(test_alert.dismissed_by)
        self.assertIsNone(test_alert.dismissed_at)
        self.assertIsNone(test_alert.dismissed_reason)
        self.assertIsNone(test_alert.dismissed_comment)
        self.assertEqual(test_alert.rule.id, "actions/missing-workflow-permissions")
        self.assertEqual(test_alert.rule.severity, "warning")
        self.assertEqual(test_alert.rule.description, "Workflow does not contain permissions")
        self.assertEqual(test_alert.rule.name, "actions/missing-workflow-permissions")
        self.assertEqual(test_alert.rule.tags[0], "actions")
        self.assertEqual(test_alert.rule.tags[1], "external/cwe/cwe-275")
        self.assertEqual(test_alert.rule.tags[2], "maintainability")
        self.assertEqual(test_alert.rule.tags[3], "security")
        self.assertEqual(
            test_alert.rule.full_description,
            "Workflows should contain explicit permissions to restrict the scope of the default GITHUB_TOKEN.",
        )
        self.assertTrue(test_alert.rule.help.startswith("## Overview"))
        self.assertEqual(test_alert.rule.security_severity_level, "medium")
        self.assertEqual(test_alert.tool.name, "CodeQL")
        self.assertIsNone(test_alert.tool.guid)
        self.assertEqual(test_alert.tool.version, "2.22.4")
        self.assertEqual(test_alert.most_recent_instance.ref, "refs/heads/main")
        self.assertEqual(test_alert.most_recent_instance.analysis_key, ".github/workflows/codeql.yml:analyze")
        self.assertEqual(test_alert.most_recent_instance.environment, '{"build-mode":"none","language":"actions"}')
        self.assertEqual(test_alert.most_recent_instance.category, "/language:actions")
        self.assertEqual(test_alert.most_recent_instance.state, "fixed")
        self.assertEqual(test_alert.most_recent_instance.commit_sha, "908396804d41cdb1d0c0538b97f25a81383ee61b")
        self.assertEqual(
            test_alert.most_recent_instance.message["text"],
            "Actions job or workflow does not limit the permissions of the GITHUB_TOKEN. Consider setting an explicit permissions block, using the following as a minimal starting point: {{contents: read}}",
        )
        self.assertEqual(test_alert.most_recent_instance.location.path, ".github/workflows/lint.yml")
        self.assertEqual(test_alert.most_recent_instance.location.start_line, 12)
        self.assertEqual(test_alert.most_recent_instance.location.end_line, 29)
        self.assertEqual(test_alert.most_recent_instance.location.start_column, 5)
        self.assertEqual(test_alert.most_recent_instance.location.end_column, 3)
        self.assertEqual(len(test_alert.most_recent_instance.classifications), 0)
        self.assertEqual(
            test_alert.instances_url,
            "https://api.github.com/repos/matt-davis27/PyGithub/code-scanning/alerts/1/instances",
        )
        self.assertIsNone(test_alert.dismissal_approved_by)

    def testRepr(self):
        alert = self.repo.get_codescan_alert(1)
        self.assertEqual(repr(alert), 'CodeScanAlert(number=1, id="actions/missing-workflow-permissions")')

    def testGetAlertsWithArguments(self):
        alerts = self.repo.get_codescan_alerts(
            tool_name="CodeQL", ref="refs/heads/main", sort="created", direction="asc", state="fixed", severity="medium"
        )
        # Note, this doesn't test "tool_guid" or "pr" arguments as they are not found in the PyGithub repo test data
        self.assertEqual(
            len(list(alerts)), 10
        )  # Update this when more alerts are added to the PyGithub repo and marked as fixed



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2017 Simon <spam@esemi.ru>                                         #
# Copyright 2018 Laurent Raufaste <analogue@glop.org>                          #
# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2024 Ramiro Morales <ramiro@users.noreply.github.com>              #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

from . import Framework


class RepositoryKey(Framework.TestCase):
    def setUp(self):
        super().setUp()
        # When recording test, be sure to create a deploy key for yourself on
        # Github and update it here.
        repo = self.g.get_user("lra").get_repo("mackup")
        self.key = repo.get_key(21870881)
        self.yet_unused_key = repo.get_key(98051552)

    def testAttributes(self):
        self.assertEqual(self.key.added_by, "key-admin-user")
        self.assertEqual(self.key.created_at, datetime(2017, 2, 22, 8, 16, 23, tzinfo=timezone.utc))
        self.assertIsNone(self.key.enabled)
        self.assertEqual(self.key.id, 21870881)
        self.assertEqual(
            self.key.key,
            "ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQDLOoLSVPwG1OSgVSeEXNbfIofYdxR5zs3u4PryhnamfFPYwi2vZW3ZxeI1oRcDh2VEdwGvlN5VUduKJNoOWMVzV2jSyR8CeDHH+I0soQCC7kfJVodU96HcPMzZ6MuVwSfD4BFGvKMXyCnBUqzo28BGHFwVQG8Ya9gL6/cTbuWywgM4xaJgMHv1OVcESXBtBkrqOneTJuOgeEmP0RfUnIAK/3/wbg9mfiBq7JV4cmWAg1xNE8GJoAbci59Tdx1dQgVuuqdQGk5jzNusOVneyMtGEB+p7UpPLJsGBW29rsMt7ITUbXM/kl9v11vPtWb+oOUThoFsDYmsWy7fGGP9YAFB",
        )
        self.assertEqual(self.key.last_used, datetime(2024, 4, 13, 10, 0, 21, tzinfo=timezone.utc))
        self.assertEqual(self.key.read_only, True)
        self.assertEqual(self.key.title, "PyGithub Test Key")
        self.assertEqual(self.key.url, "https://api.github.com/repos/lra/mackup/keys/21870881")
        self.assertEqual(
            self.key.created_at,
            datetime(2017, 2, 22, 8, 16, 23, tzinfo=timezone.utc),
        )
        self.assertTrue(self.key.verified)
        self.assertTrue(self.key.read_only)
        self.assertEqual(repr(self.key), 'RepositoryKey(title="PyGithub Test Key", id=21870881)')
        self.assertEqual(self.key.added_by, "key-admin-user")
        self.assertEqual(
            self.key.last_used,
            datetime(2024, 4, 13, 10, 0, 21, tzinfo=timezone.utc),
        )

    def testLazyAttributes(self):
        issue = self.g.withLazy(True).get_repo("lazy/repo").get_key(42)
        self.assertEqual(str(issue), "RepositoryKey(title=None, id=42)")
        self.assertEqual(issue.id, 42)
        self.assertEqual(issue.url, "/repos/lazy/repo/keys/42")

    def testYetUnusedKey(self):
        self.assertEqual(self.yet_unused_key.id, 98051552)
        self.assertEqual(
            self.yet_unused_key.key,
            "ssh-ed25519 AAAAC3NzaC1lZDI1NTE5AAAAIOXEPB9eqJ2DwZepFxrPwCDczIReVeWOOt3NMs8KOn3h",
        )
        self.assertEqual(self.yet_unused_key.last_used, None)

    def testDelete(self):
        self.key.delete()



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Martijn Koster <mak-github@greenhills.co.uk>                  #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2017 Simon <spam@esemi.ru>                                         #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2024 Kian-Meng Ang <kianmeng.ang@gmail.com>                        #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

from . import Framework


class CommitStatus(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.statuses = list(
            self.g.get_user().get_repo("PyGithub").get_commit("1292bf0e22c796e91cc3d6e24b544aece8c21f2a").get_statuses()
        )
        self.status = self.statuses[0]

    def testAttributes(self):
        self.assertIsNone(self.status.avatar_url)
        self.assertEqual(self.status.context, "build")
        self.assertEqual(
            self.status.created_at,
            datetime(2012, 9, 8, 11, 30, 56, tzinfo=timezone.utc),
        )
        self.assertEqual(self.status.creator.login, "jacquev6")
        self.assertEqual(self.status.description, "Status successfully created by PyGithub")
        self.assertEqual(self.status.id, 277040)
        self.assertIsNone(self.status.node_id)
        self.assertIsNone(self.status.required)
        self.assertEqual(self.status.state, "success")
        self.assertEqual(self.status.target_url, "https://github.com/jacquev6/PyGithub/issues/67")
        self.assertEqual(
            self.status.updated_at,
            datetime(2012, 9, 8, 11, 30, 56, tzinfo=timezone.utc),
        )
        self.assertEqual(self.status.creator.login, "jacquev6")
        self.assertEqual(self.status.description, "Status successfully created by PyGithub")
        self.assertEqual(self.statuses[1].description, None)
        self.assertEqual(self.status.id, 277040)
        self.assertEqual(self.status.state, "success")
        self.assertEqual(self.statuses[1].state, "pending")
        self.assertEqual(self.status.context, "build")
        self.assertEqual(
            self.status.target_url,
            "https://github.com/jacquev6/PyGithub/issues/67",
        )
        self.assertEqual(self.statuses[1].target_url, None)
        self.assertEqual(
            repr(self.status),
            'CommitStatus(state="success", id=277040, context="build")',
        )
        self.assertEqual(self.status.url, "https://api.github.com/repos/jacquev6/PyGithub/statuses/277040")



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2017 Simon <spam@esemi.ru>                                         #
# Copyright 2018 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2024 Benjamin K <53038537+treee111@users.noreply.github.com>       #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from . import Framework


class RequiredPullRequestReviews(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.required_pull_request_reviews = (
            self.g.get_user().get_repo("PyGithub").get_branch("integrations").get_required_pull_request_reviews()
        )

    def testAttributes(self):
        self.assertIsNone(self.required_pull_request_reviews.bypass_pull_request_allowances)
        self.assertTrue(self.required_pull_request_reviews.dismiss_stale_reviews)
        self.assertIsNone(self.required_pull_request_reviews.dismissal_restrictions)
        self.assertTrue(self.required_pull_request_reviews.require_code_owner_reviews)
        self.assertIsNone(self.required_pull_request_reviews.require_last_push_approval)
        self.assertEqual(self.required_pull_request_reviews.required_approving_review_count, 3)
        self.assertEqual(
            self.required_pull_request_reviews.url,
            "https://api.github.com/repos/jacquev6/PyGithub/branches/integrations/protection/required_pull_request_reviews",
        )
        self.assertIs(self.required_pull_request_reviews.dismissal_users, None)
        self.assertIs(self.required_pull_request_reviews.dismissal_teams, None)
        self.assertEqual(
            self.required_pull_request_reviews.__repr__(),
            'RequiredPullRequestReviews(url="https://api.github.com/repos/jacquev6/PyGithub/branches/integrations/protection/required_pull_request_reviews", require_last_push_approval=None, require_code_owner_reviews=True, dismiss_stale_reviews=True)',
        )

    def testOrganizationOwnedTeam(self):
        required_pull_request_reviews = (
            self.g.get_repo("PyGithub/PyGithub", lazy=True)
            .get_branch("integrations")
            .get_required_pull_request_reviews()
        )
        self.assertIsNone(required_pull_request_reviews.bypass_pull_request_allowances.apps)
        self.assertListKeyEqual(
            required_pull_request_reviews.bypass_pull_request_allowances.users,
            lambda u: u.login,
            ["jacquev6"],
        )
        self.assertListKeyEqual(
            required_pull_request_reviews.bypass_pull_request_allowances.teams,
            lambda t: t.slug,
            ["pygithub-owners"],
        )

        self.assertIsNone(required_pull_request_reviews.dismissal_restrictions.apps)
        self.assertListKeyEqual(
            required_pull_request_reviews.dismissal_restrictions.users,
            lambda u: u.login,
            ["jacquev6"],
        )
        self.assertListKeyEqual(
            required_pull_request_reviews.dismissal_restrictions.teams,
            lambda t: t.slug,
            ["pygithub-owners"],
        )
        self.assertEqual(
            required_pull_request_reviews.dismissal_restrictions.users_url,
            "https://api.github.com/repos/PyGithub/PyGithub/branches/integrations/protection/dismissal_restrictions/users",
        )
        self.assertEqual(
            required_pull_request_reviews.dismissal_restrictions.teams_url,
            "https://api.github.com/repos/PyGithub/PyGithub/branches/integrations/protection/dismissal_restrictions/teams",
        )

        self.assertListKeyEqual(
            required_pull_request_reviews.dismissal_users,
            lambda u: u.login,
            ["jacquev6"],
        )
        self.assertListKeyEqual(
            required_pull_request_reviews.dismissal_teams,
            lambda t: t.slug,
            ["pygithub-owners"],
        )



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 Michell Stuttgart <michellstut@gmail.com>                     #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import date, datetime, timezone

from . import Framework


class Milestone(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.milestone = self.g.get_repo("PyGithub/PyGithub").get_milestone(1)

    def testAttributes(self):
        self.assertEqual(self.milestone.closed_at, datetime(2012, 3, 12, 22, 18, 49, tzinfo=timezone.utc))
        self.assertEqual(self.milestone.closed_issues, 2)
        self.assertEqual(self.milestone.created_at, datetime(2012, 3, 8, 12, 22, 10, tzinfo=timezone.utc))
        self.assertEqual(self.milestone.creator.login, "jacquev6")
        self.assertEqual(self.milestone.description, "")
        self.assertEqual(self.milestone.due_on, datetime(2012, 3, 13, 7, 0, 0, tzinfo=timezone.utc))
        self.assertEqual(self.milestone.html_url, "https://github.com/PyGithub/PyGithub/milestone/1")
        self.assertEqual(self.milestone.id, 93546)
        self.assertEqual(
            self.milestone.labels_url, "https://api.github.com/repos/PyGithub/PyGithub/milestones/1/labels"
        )
        self.assertEqual(self.milestone.node_id, "MDk6TWlsZXN0b25lOTM1NDY=")
        self.assertEqual(self.milestone.number, 1)
        self.assertEqual(self.milestone.open_issues, 0)
        self.assertEqual(self.milestone.state, "closed")
        self.assertEqual(self.milestone.title, "Version 0.4")
        self.assertEqual(self.milestone.updated_at, datetime(2012, 9, 11, 18, 48, 34, tzinfo=timezone.utc))
        self.assertEqual(self.milestone.url, "https://api.github.com/repos/PyGithub/PyGithub/milestones/1")
        self.assertEqual(self.milestone.creator.login, "jacquev6")
        self.assertEqual(repr(self.milestone), 'Milestone(title="Version 0.4", number=1)')

    def testLazyAttributes(self):
        milestone = self.g.withLazy(True).get_repo("lazy/repo").get_milestone(42)
        self.assertEqual(str(milestone), "Milestone(title=None, number=42)")
        self.assertEqual(milestone._identity, 42)
        self.assertEqual(milestone.number, 42)
        self.assertEqual(milestone.url, "/repos/lazy/repo/milestones/42")

    def testEditWithMinimalParameters(self):
        self.milestone.edit("Title edited by PyGithub")
        self.assertEqual(self.milestone.title, "Title edited by PyGithub")

    def testEditWithAllParameters(self):
        self.milestone.edit(
            "Title edited twice by PyGithub",
            "closed",
            "Description edited by PyGithub",
            due_on=date(2012, 6, 16),
        )
        self.assertEqual(self.milestone.title, "Title edited twice by PyGithub")
        self.assertEqual(self.milestone.state, "closed")
        self.assertEqual(self.milestone.description, "Description edited by PyGithub")
        self.assertEqual(
            self.milestone.due_on,
            datetime(2012, 6, 16, 7, 0, 0, tzinfo=timezone.utc),
        )

    def testGetLabels(self):
        self.assertListKeyEqual(
            self.milestone.get_labels(),
            lambda lb: lb.name,
            ["Public interface", "Project management"],
        )

    def testDelete(self):
        self.milestone.delete()



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

import github.NamedUser

from . import Framework


class RawData(Framework.TestCase):
    jacquev6RawData = {
        "disk_usage": 13812,
        "private_gists": 5,
        "public_repos": 21,
        "subscriptions_url": "https://api.github.com/users/jacquev6/subscriptions",
        "gravatar_id": "b68de5ae38616c296fa345d2b9df2225",
        "hireable": False,
        "id": 327146,
        "followers_url": "https://api.github.com/users/jacquev6/followers",
        "following_url": "https://api.github.com/users/jacquev6/following",
        "collaborators": 1,
        "total_private_repos": 4,
        "blog": "http://vincent-jacques.net",
        "followers": 22,
        "location": "Paris, France",
        "type": "User",
        "email": "vincent@vincent-jacques.net",
        "bio": "",
        "gists_url": "https://api.github.com/users/jacquev6/gists{/gist_id}",
        "owned_private_repos": 4,
        "company": "Criteo",
        "events_url": "https://api.github.com/users/jacquev6/events{/privacy}",
        "html_url": "https://github.com/jacquev6",
        "updated_at": "2013-03-12T22:13:32Z",
        "plan": {
            "collaborators": 1,
            "name": "micro",
            "private_repos": 5,
            "space": 614400,
        },
        "received_events_url": "https://api.github.com/users/jacquev6/received_events",
        "starred_url": "https://api.github.com/users/jacquev6/starred{/owner}{/repo}",
        "public_gists": 2,
        "name": "Vincent Jacques",
        "organizations_url": "https://api.github.com/users/jacquev6/orgs",
        "url": "https://api.github.com/users/jacquev6",
        "created_at": "2010-07-09T06:10:06Z",
        "avatar_url": "https://secure.gravatar.com/avatar/b68de5ae38616c296fa345d2b9df2225?d=https://a248.e.akamai.net/assets.github.com%2Fimages%2Fgravatars%2Fgravatar-user-420.png",
        "repos_url": "https://api.github.com/users/jacquev6/repos",
        "following": 38,
        "login": "jacquev6",
    }

    planRawData = {
        "collaborators": 1,
        "name": "micro",
        "private_repos": 5,
        "space": 614400,
    }

    def testCompletedObject(self):
        user = self.g.get_user("jacquev6")
        self.assertTrue(user._CompletableGithubObject__completed)
        self.assertEqual(user.raw_data, RawData.jacquev6RawData)

    def testNotYetCompletedObject(self):
        user = self.g.get_user().get_repo("PyGithub").owner
        self.assertFalse(user._CompletableGithubObject__completed)
        self.assertEqual(user.raw_data, RawData.jacquev6RawData)
        self.assertTrue(user._CompletableGithubObject__completed)

    def testNonCompletableObject(self):
        plan = self.g.get_user().plan
        self.assertEqual(plan.raw_data, RawData.planRawData)

    def testCreateObjectFromRawData(self):
        user = self.g.create_from_raw_data(github.NamedUser.NamedUser, RawData.jacquev6RawData)
        self.assertEqual(user._CompletableGithubObject__completed, True)
        self.assertEqual(user.name, "Vincent Jacques")



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2021 Claire Johns <42869556+johnsc1@users.noreply.github.com>      #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

from . import Framework


class IssueType(Framework.TestCase):
    def setUp(self):
        super().setUp()
        with self.replayData("Issue.setUp.txt"):
            self.repo = self.g.get_repo("PyGithub/PyGithub")
            self.issue = self.repo.get_issue(28)
            self.type = self.issue.type

    def testAttributes(self):
        self.assertEqual(self.type.color, "red")
        self.assertEqual(self.type.created_at, datetime(2024, 1, 25, 12, 55, 41, tzinfo=timezone.utc))
        self.assertEqual(self.type.description, "An unexpected problem or behavior")
        self.assertEqual(self.type.id, 1386163)
        self.assertEqual(self.type.is_enabled, True)
        self.assertEqual(self.type.name, "Bug")
        self.assertEqual(self.type.node_id, "IT_kwDOAKxBpM4AFSaz")
        self.assertEqual(self.type.updated_at, datetime(2024, 7, 26, 10, 24, 51, tzinfo=timezone.utc))



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2020 Florent Clarret <florent.clarret@gmail.com>                   #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

from . import Framework


class ProjectColumn(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.get_project_column = self.g.get_project_column(8700460)
        self.move_project_column = self.g.get_project_column(8748065)
        proj = self.g.get_project(1682941)
        self.col = proj.get_columns()[0]

    # See https://developer.github.com/v3/projects/columns/#get-a-project-column
    def testAttributes(self):
        col = self.col
        self.assertEqual(col.cards_url, "https://api.github.com/projects/columns/3138830/cards")
        self.assertEqual(col.created_at.year, 2018)
        self.assertEqual(col.id, 3138830)
        self.assertEqual(col.name, "To Do")
        self.assertEqual(col.node_id, "MDEzOlByb2plY3RDb2x1bW4zMTM4ODMw")
        self.assertEqual(col.project_url, "https://api.github.com/projects/1682941")
        self.assertEqual(col.updated_at, datetime(2018, 8, 1, 4, 7, 35, tzinfo=timezone.utc))
        self.assertEqual(col.url, "https://api.github.com/projects/columns/3138830")
        self.assertEqual(repr(col), 'ProjectColumn(name="To Do")')

    def testCreate(self):
        repo = self.g.get_user().get_repo("PyGithub")
        project = repo.create_project("Project created by PyGithub", "Project Body")
        column = project.create_column(
            "Project Column created by PyGithub",
        )
        self.assertEqual(column.id, 3999333)

    def testGetProjectColumn(self):
        self.assertEqual(self.get_project_column.id, 8700460)
        self.assertEqual(self.get_project_column.name, "c1")
        self.assertEqual(
            self.get_project_column.cards_url,
            "https://api.github.com/projects/columns/8700460/cards",
        )
        self.assertEqual(self.get_project_column.node_id, "MDEzOlByb2plY3RDb2x1bW44NzAwNDYw")
        self.assertEqual(
            self.get_project_column.project_url,
            "https://api.github.com/projects/4294766",
        )
        self.assertEqual(
            self.get_project_column.url,
            "https://api.github.com/projects/columns/8700460",
        )
        self.assertEqual(
            self.get_project_column.created_at,
            datetime(2020, 4, 13, 20, 29, 53, tzinfo=timezone.utc),
        )
        self.assertEqual(
            self.get_project_column.updated_at,
            datetime(2020, 4, 14, 18, 9, 38, tzinfo=timezone.utc),
        )

    def testGetAllCards(self):
        cards = self.get_project_column.get_cards(archived_state="all")
        self.assertEqual(cards.totalCount, 3)
        self.assertEqual(cards[0].id, 36285184)
        self.assertEqual(cards[0].note, "Note3")
        self.assertEqual(cards[1].id, 36281526)
        self.assertEqual(cards[1].note, "Note2")
        self.assertEqual(cards[2].id, 36281516)
        self.assertEqual(cards[2].note, "Note1")

    def testGetArchivedCards(self):
        cards = self.get_project_column.get_cards(archived_state="archived")
        self.assertEqual(cards.totalCount, 1)
        self.assertEqual(cards[0].id, 36281516)
        self.assertEqual(cards[0].note, "Note1")

    def testGetNotArchivedCards(self):
        cards = self.get_project_column.get_cards(archived_state="not_archived")
        self.assertEqual(cards.totalCount, 2)
        self.assertEqual(cards[0].id, 36285184)
        self.assertEqual(cards[0].note, "Note3")
        self.assertEqual(cards[1].id, 36281526)
        self.assertEqual(cards[1].note, "Note2")

    def testGetCards(self):
        cards = self.get_project_column.get_cards()
        self.assertEqual(cards.totalCount, 2)
        self.assertEqual(cards[0].id, 36285184)
        self.assertEqual(cards[0].note, "Note3")
        self.assertEqual(cards[1].id, 36281526)
        self.assertEqual(cards[1].note, "Note2")

    def testCreateCard(self):
        new_card = self.get_project_column.create_card(note="NewCard")
        self.assertEqual(new_card.id, 36290228)
        self.assertEqual(new_card.note, "NewCard")

    def testDelete(self):
        project_column = self.g.get_project_column(8747987)
        self.assertTrue(project_column.delete())

    def testEdit(self):
        self.move_project_column.edit("newTestColumn")
        self.assertEqual(self.move_project_column.id, 8748065)
        self.assertEqual(self.move_project_column.name, "newTestColumn")

    def testMoveFirst(self):
        self.assertTrue(self.move_project_column.move(position="first"))

    def testMoveLast(self):
        self.assertTrue(self.move_project_column.move(position="last"))

    def testMoveAfter(self):
        self.assertTrue(self.move_project_column.move(position="after:8700460"))



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from typing import Any

import github
import github.GithubException
import github.Organization
import github.Repository
import github.RepositoryDiscussion
import github.RepositoryDiscussionComment
import github.Requester
from github import Github

from . import Framework


class GraphQl(Framework.TestCase):
    def setUp(self):
        super().setUp()

    def expected(self, base_url: str = "https://github.com") -> dict[Any, Any]:
        return {
            "actor": {
                "avatarUrl": "https://avatars.githubusercontent.com/u/14806300?u=786f9f8ef8782d45381b01580f7f7783cf9c7e37&v=4",
                "login": "heitorpolidoro",
                "resourcePath": "/heitorpolidoro",
                "url": f"{base_url}/heitorpolidoro",
            },
            "clientMutationId": None,
        }

    def testRequesterGraphQlPrefix(self):
        get_graphql_prefix = github.Requester.Requester.get_graphql_prefix
        assert "/graphql" == get_graphql_prefix(None)
        assert "/graphql" == get_graphql_prefix("")
        assert "/graphql" == get_graphql_prefix("/")
        assert "/api/graphql" == get_graphql_prefix("/api/v3")
        assert "/path/to/github/api/graphql" == get_graphql_prefix("/path/to/github/api/v3")
        assert "/path/to/github/graphql" == get_graphql_prefix("/path/to/github")

    def testDefaultUrl(self):
        pull = self.g.get_repo("PyGithub/PyGithub").get_pull(31)
        response = pull.disable_automerge()
        assert response == self.expected()

    def testOtherUrl(self):
        base_url = "https://my.enterprise.com/api/v3"
        gh = Github(base_url=base_url)
        pull = gh.get_repo("PyGithub/PyGithub").get_pull(31)
        response = pull.disable_automerge()
        assert response == self.expected(base_url)

    def testOtherPort(self):
        base_url = "https://my.enterprise.com:8080/api/v3"
        gh = Github(base_url=base_url)
        pull = gh.get_repo("PyGithub/PyGithub").get_pull(31)
        response = pull.disable_automerge()
        assert response == self.expected(base_url)

    def testNode(self):
        requester = self.g._Github__requester
        headers, data = requester.graphql_node("D_kwDOADYVqs4ATJZD", "title", "Discussion")
        self.assertTrue(headers)
        self.assertEqual(
            data.get("data", {}).get("node", {}).get("title"),
            "Is there a way to search if a string present in default branch?",
        )

        # non-existing node should throw a NOT FOUND exception
        with self.assertRaises(github.UnknownObjectException) as e:
            requester.graphql_node("D_abcdefgh", "title", "Discussion")
        self.assertEqual(e.exception.status, 404)
        self.assertEqual(
            e.exception.data,
            {
                "data": {"node": None},
                "errors": [
                    {
                        "type": "NOT_FOUND",
                        "path": ["node"],
                        "locations": [{"line": 3, "column": 15}],
                        "message": "Could not resolve to a node with the global id of 'D_abcdefgh'",
                    }
                ],
            },
        )
        self.assertEqual(e.exception.message, "Could not resolve to a node with the global id of 'D_abcdefgh'")

        # wrong type should throw an exception
        with self.assertRaises(github.GithubException) as e:
            requester.graphql_node("D_kwDOADYVqs4ATJZD", "login", "User")
        self.assertEqual(e.exception.message, "Retrieved User object is of different type: Discussion")
        self.assertEqual(e.exception.status, 400)
        self.assertEqual(e.exception.data, {"data": {"node": {"__typename": "Discussion"}}})

    def testNodeClass(self):
        requester = self.g._Github__requester
        discussion = requester.graphql_node_class(
            "D_kwDOADYVqs4ATJZD", "title", github.RepositoryDiscussion.RepositoryDiscussion, "Discussion"
        )
        self.assertEqual(discussion.title, "Is there a way to search if a string present in default branch?")

        # non-existing node should throw a NOT FOUND exception
        with self.assertRaises(github.UnknownObjectException) as e:
            requester.graphql_node_class(
                "D_abcdefgh", "title", github.RepositoryDiscussion.RepositoryDiscussion, "Discussion"
            )
        self.assertEqual(e.exception.status, 404)
        self.assertEqual(
            e.exception.data,
            {
                "data": {"node": None},
                "errors": [
                    {
                        "type": "NOT_FOUND",
                        "path": ["node"],
                        "locations": [{"line": 3, "column": 15}],
                        "message": "Could not resolve to a node with the global id of 'D_abcdefgh'",
                    }
                ],
            },
        )
        self.assertEqual(e.exception.message, "Could not resolve to a node with the global id of 'D_abcdefgh'")

        # wrong type should throw an exception
        with self.assertRaises(github.GithubException) as e:
            requester.graphql_node_class(
                "D_kwDOADYVqs4ATJZD", "login", github.RepositoryDiscussion.RepositoryDiscussion, "User"
            )
        self.assertEqual(e.exception.message, "Retrieved User object is of different type: Discussion")
        self.assertEqual(e.exception.status, 400)
        self.assertEqual(e.exception.data, {"data": {"node": {"__typename": "Discussion"}}})

    def testQuery(self):
        requester = self.g._Github__requester
        query = """
            query Q($owner: String!, $name: String!) {
              repository(owner: $owner, name: $name) { url }
            }"""
        variables = {"owner": "PyGithub", "name": "PyGithub"}
        header, data = requester.graphql_query(query, variables)
        self.assertTrue(header)
        self.assertEqual(data, {"data": {"repository": {"url": "https://github.com/PyGithub/PyGithub"}}})

    def testQueryRestClass(self):
        requester = self.g._Github__requester
        query = """
            query Q($owner: String!, $name: String!) {
              repository(owner: $owner, name: $name) { url }
            }"""
        variables = {"owner": "PyGithub", "name": "PyGithub"}
        repo = requester.graphql_query_class(query, variables, ["repository"], github.Repository.Repository)
        self.assertIsInstance(repo, github.Repository.Repository)
        self.assertEqual(repo.html_url, "https://github.com/PyGithub/PyGithub")

    def testQueryGraphQlClass(self):
        requester = self.g._Github__requester
        query = """
            query Q($id: ID!) {
              node(id: $id) { ... on DiscussionComment { url } }
            }"""
        variables = {"id": "DC_kwDOADYVqs4AU3Mg"}
        comment = requester.graphql_query_class(
            query, variables, ["node"], github.RepositoryDiscussionComment.RepositoryDiscussionComment
        )
        self.assertIsInstance(comment, github.RepositoryDiscussionComment.RepositoryDiscussionComment)
        self.assertEqual(
            comment.html_url, "https://github.com/PyGithub/PyGithub/discussions/2480#discussioncomment-5468960"
        )

    def testMutation(self):
        requester = self.g._Github__requester
        header, data = requester.graphql_named_mutation(
            "followOrganization", {"organizationId": "O_kgDOAKxBpA"}, "organization { name }"
        )
        self.assertTrue(header)
        self.assertEqual(data, {"organization": {"name": "PyGithub"}})

    def testMutationClass(self):
        requester = self.g._Github__requester
        org = requester.graphql_named_mutation_class(
            "followOrganization",
            {"organizationId": "O_kgDOAKxBpA"},
            "organization { name }",
            "organization",
            github.Organization.Organization,
        )
        self.assertEqual(org.name, "PyGithub")

    def testPaginationAndRestIntegration(self):
        repo = self.g.get_repo("PyGithub/PyGithub")
        discussion_schema = """
            id
            url
            number
            author {
              login
              avatarUrl
              url
            }
            repository {
              owner { login }
              name
              issues(first: 10) {
                totalCount
                pageInfo {
                  startCursor
                  endCursor
                  hasNextPage
                  hasPreviousPage
                }
                nodes {
                  databaseId
                  id
                  number
                  title
                }
              }
            }
            title
            createdAt
            comments(first: 10) {
              totalCount
              pageInfo {
                startCursor
                endCursor
                hasNextPage
                hasPreviousPage
              }
              nodes {
                id
                url
                createdAt
                author {
                  login
                  avatarUrl
                  url
                }
                isAnswer
                replies(first: 10) {
                  totalCount
                  pageInfo {
                    startCursor
                    endCursor
                    hasNextPage
                    hasPreviousPage
                  }
                  nodes {
                    id
                    url
                    createdAt
                    author {
                      login
                      avatarUrl
                      url
                    }
                  }
                }
              }
            }
            labels(first: 10) {
              totalCount
              pageInfo {
                startCursor
                endCursor
                hasNextPage
                hasPreviousPage
              }
              nodes {
                id
                name
                issues(first: 10) {
                  totalCount
                  pageInfo {
                    startCursor
                    endCursor
                    hasNextPage
                    hasPreviousPage
                  }
                  nodes {
                    databaseId
                    id
                    number
                    title
                  }
                }
              }
            }
          """
        discussions_pages = repo.get_discussions(discussion_schema)
        discussions = list(discussions_pages)
        # would perform an extra request if called before iterating discussions_pages
        self.assertEqual(discussions_pages.totalCount, 65)
        self.assertEqual(len(discussions), 65)
        self.assertEqual(discussions[0].number, 3044)
        self.assertEqual(discussions[-1].number, 1780)

        discussion = discussions[28]
        self.assertEqual(discussion.author.login, "arunanandhan")
        self.assertEqual(discussion.node_id, "D_kwDOADYVqs4ATJZD")
        self.assertEqual(
            discussion.author.avatar_url,
            "https://avatars.githubusercontent.com/u/48812131?u=571c345a5994a55100a16b45a9688f5d6d340730&v=4",
        )
        self.assertEqual(discussion.author.html_url, "https://github.com/arunanandhan")

        # inner page of GraphQL comments
        comments = discussion.get_comments("id")
        self.assertEqual(comments.totalCount, 1)  # does not perform an extra request
        comments = list(comments)
        self.assertEqual(len(comments), 1)
        comment = comments[0]
        self.assertEqual(comment.node_id, "DC_kwDOADYVqs4AU3Mg")
        self.assertEqual(
            comment.html_url, "https://github.com/PyGithub/PyGithub/discussions/2480#discussioncomment-5468960"
        )
        self.assertEqual(comment.author.login, "EnricoMi")

        # inner inner page of GraphQL replies
        replies = comment.get_replies()
        self.assertEqual(replies.totalCount, 5)  # does not perform an extra request
        self.assertEqual(replies[0].node_id, "DC_kwDOADYVqs4AU3Wg")

        # inner page of REST labels
        labels_pages = discussions[3].get_labels()
        self.assertEqual(labels_pages.totalCount, 1)
        label = labels_pages[0]
        self.assertEqual(label.name, "Call for Contribution")

        # inner REST repository
        repo = discussion.repository
        issues_pages = repo.get_issues()
        issue = issues_pages[0]
        # GraphQL retrieved 10 issues, but repo.get_issues() is not aware of that data
        # it calls the REST API
        self.assertEqual(issues_pages.totalCount, 341)
        self.assertEqual(issue.number, 3045)



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 Morgan Goose <morgan.goose@gmail.com>                         #
# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2023 Liuyang Wan <tsfdye@gmail.com>                                #
# Copyright 2025 Aidan McNay <acm289@cornell.edu>                              #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Hugo van Kemenade <1324225+hugovk@users.noreply.github.com>   #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

import datetime
import glob
import os
import re
import sys
from collections.abc import Iterable

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
sys.path.insert(0, os.path.abspath(".."))
from importlib.metadata import version  # noqa: E402

setupVersion = version("pygithub")

# -- General configuration -----------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = ["sphinx.ext.autodoc", "sphinx.ext.coverage", "sphinx.ext.mathjax"]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix of source filenames.
source_suffix = ".rst"

# The encoding of source files.
# source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = "index"

# General information about the project.
project = "PyGithub"
copyright = "%d, Vincent Jacques, Liuyang Wan, Steve Kowalik, Enrico Minack" % datetime.date.today().year

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = setupVersion
# The full version, including alpha/beta/rc tags.
release = setupVersion

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
# language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
# today = ''
# Else, today_fmt is used as the format for a strftime call.
# today_fmt = '%B %d, %Y'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = []

# The reST default role (used for this markup: `text`) to use for all documents.
# default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
# add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
# add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
# show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = "sphinx"

# A list of ignored prefixes for module index sorting.
# modindex_common_prefix = []


# -- Options for HTML output ---------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = "sphinx_rtd_theme"

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
# html_theme_options = {}

# Add any paths that contain custom themes here, relative to this directory.
# html_theme_path = []

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
# html_title = None

# A shorter title for the navigation bar.  Default is the same as html_title.
# html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
# html_logo = None

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
# html_favicon = None

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
# html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
# html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
# html_sidebars = {}

# Additional templates that should be rendered to pages, maps page names to
# template names.
# html_additional_pages = {}

# If false, no module index is generated.
# html_domain_indices = True

# If false, no index is generated.
# html_use_index = True

# If true, the index is split into individual pages for each letter.
# html_split_index = False

# If true, links to the reST sources are added to the pages.
# html_show_sourcelink = True

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
# html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
# html_show_copyright = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
# html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
# html_file_suffix = None

# Output file base name for HTML help builder.
htmlhelp_basename = "PyGithubdoc"


# -- Options for LaTeX output --------------------------------------------------

# latex_elements = {
# The paper size ('letterpaper' or 'a4paper').
# 'papersize': 'letterpaper',

# The font size ('10pt', '11pt' or '12pt').
# 'pointsize': '10pt',

# Additional stuff for the LaTeX preamble.
# 'preamble': '',
# }

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass [howto/manual]).
latex_documents = [
    ("index", "PyGithub.tex", "PyGithub Documentation", "Vincent Jacques", "manual"),
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
# latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
# latex_use_parts = False

# If true, show page references after internal links.
# latex_show_pagerefs = False

# If true, show URL addresses after external links.
# latex_show_urls = False

# Documents to append as an appendix to all manuals.
# latex_appendices = []

# If false, no module index is generated.
# latex_domain_indices = True


# -- Options for manual page output --------------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [("index", "pygithub", "PyGithub Documentation", ["Vincent Jacques"], 1)]

# If true, show URL addresses after external links.
# man_show_urls = False


# -- Options for Texinfo output ------------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (
        "index",
        "PyGithub",
        "PyGithub Documentation",
        "Vincent Jacques",
        "PyGithub",
        "One line description of project.",
        "Miscellaneous",
    ),
]

# Documents to append as an appendix to all manuals.
# texinfo_appendices = []

# If false, no module index is generated.
# texinfo_domain_indices = True

# How to display URL addresses: 'footnote', 'no', or 'inline'.
# texinfo_show_urls = 'footnote'

autodoc_default_options = {"members": True}
autodoc_member_order = "bysource"
autoclass_content = "both"

githubObjectTypes = {
    variation
    for object_type in ["GithubObject", "CompletableGithubObject", "NonCompletableGithubObject"]
    for variation in [object_type, "GithubObject." + object_type, "github.GithubObject." + object_type]
}
githubObjectClasses: dict[str, str] = {}


def collect_classes(types: set[str]) -> Iterable[tuple[str, str]]:
    def get_base_classes(class_definition: str) -> Iterable[str]:
        if "(" in class_definition and ")" in class_definition:
            for base in class_definition[class_definition.index("(") + 1 : class_definition.index(")")].split(","):
                yield base.strip()
        else:
            return []

    for filename in sorted(glob.glob("../github/*.py")):
        module = f"github.{filename[10:-3]}"
        with open(filename) as r:
            for line in r.readlines():
                if line.startswith("class ") and any([base in types for base in get_base_classes(line)]):
                    class_name = re.match(r"class (\w+)[:(]", line).group(1)
                    if class_name not in types:
                        yield class_name, f"{module}"


# get all classes derived from GithubObject classes directly
classes = list(collect_classes(githubObjectTypes))
while classes:
    githubObjectClasses.update(classes)
    # get all classes derived from detected classes
    githubObjectTypes.update(
        {
            variation
            for object_type in [cls for cls, _ in classes]
            for variation in [object_type, "GithubObject." + object_type, "github.GithubObject." + object_type]
        }
    )
    classes = list(collect_classes(set(githubObjectTypes)))

with open("github_objects.rst", "w") as f:
    f.write("Github objects\n")
    f.write("==============\n")
    f.write("\n")
    f.write(".. autoclass:: github.GithubObject.GithubObject()\n")
    f.write("\n")
    f.write(".. toctree::\n")
    for githubObjectClass in sorted(githubObjectClasses.keys()):
        f.write("   github_objects/" + githubObjectClass + "\n")

for githubObjectClass, module in githubObjectClasses.items():
    with open("github_objects/" + githubObjectClass + ".rst", "w") as f:
        f.write(githubObjectClass + "\n")
        f.write("=" * len(githubObjectClass) + "\n")
        f.write("\n")
        f.write(".. autoclass:: " + module + "." + githubObjectClass + "()\n")

methods = dict()
githubObjectClasses.update([("MainClass", "github.MainClass")])
githubObjectClasses.update([("GithubIntegration", "github.GithubIntegration")])
for githubObjectClass, module in githubObjectClasses.items():
    with open("../" + module.replace(".", "/") + ".py") as f:
        method = None
        isProperty = False
        for line in f:
            line = line.rstrip()
            if line == "    @property":
                isProperty = True
            if line.startswith("    def "):
                if not isProperty:
                    method = line.split("(")[0][8:]
                    if method in [
                        "_initAttributes",
                        "_useAttributes",
                        "__init__",
                        "__create_pull_1",
                        "__create_pull_2",
                        "__create_pull",
                        "_hub",
                        "__get_FIX_REPO_GET_GIT_REF",
                        "__set_FIX_REPO_GET_GIT_REF",
                        "__get_per_page",
                        "__set_per_page",
                        "create_from_raw_data",
                        "dump",
                        "load",
                    ]:
                        method = None
                isProperty = False
            if line.startswith("        :calls: `") and method:
                for callee in line[16:].split(" or "):
                    verb, url = callee[1:].split(" ")[0:2]
                    if url not in methods:
                        methods[url] = dict()
                    if verb not in methods[url]:
                        methods[url][verb] = set()
                    methods[url][verb].add(":meth:`" + module + "." + githubObjectClass + "." + method + "`")

methods["/markdown/raw"] = dict()
methods["/markdown/raw"]["POST"] = ["Not implemented, see ``/markdown``"]
methods["/rate_limit"] = dict()
methods["/rate_limit"]["GET"] = ["Not implemented, see `Github.rate_limiting`"]

with open("apis.rst", "w") as apis:
    apis.write("APIs\n")
    apis.write("====\n")
    apis.write("\n")
    for url, verbs in sorted(methods.items()):
        apis.write("* ``" + url + "``\n")
        apis.write("\n")
        for verb in ["GET", "PATCH", "POST", "PUT", "DELETE"]:
            if verb in verbs:
                apis.write("  * " + verb + ": " + " or ".join(sorted(verbs[verb])) + "\n")
        apis.write("\n")



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2013 davidbrai <davidbrai@gmail.com>                               #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2015 Eliot Walker <eliot@lyft.com>                                 #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2021 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2022 Liuyang Wan <tsfdye@gmail.com>                                #
# Copyright 2023 Andrew Dawes <53574062+AndrewJDawes@users.noreply.github.com> #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2023 YugoHino <henom06@gmail.com>                                  #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2024 Kian-Meng Ang <kianmeng.ang@gmail.com>                        #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Matej Focko <mfocko@users.noreply.github.com>                 #
# Copyright 2025 odedperezcodes <oded.perez.codes@gmail.com>                   #
# Copyright 2026 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from datetime import datetime, timezone

from github.PaginatedList import PaginatedList as PaginatedListImpl

from . import Framework


class PaginatedList(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.repo = self.g.get_user("openframeworks").get_repo("openFrameworks")
        self.list = self.repo.get_issues()
        self.licenses = self.g.get_enterprise("beaver-group").get_consumed_licenses()

    def testIsApiType(self):
        self.assertTrue(self.list.is_rest)
        self.assertFalse(self.list.is_graphql)

    def testIteration(self):
        self.assertEqual(len(list(self.list)), 333)

    def testIterationWithPrefetchedFirstPage(self):
        # test data taken from EnterpriseAdmin.testGetEnterpriseUsers
        users = self.licenses.users
        self.assertEqual(len(list(users)), 102)
        self.assertEqual(len({user.github_com_login for user in users}), 102)

    def testSeveralIterations(self):
        with self.replayData("PaginatedList.testIteration.txt"):
            self.assertEqual(len(list(self.list)), 333)
            self.assertEqual(len(list(self.list)), 333)
            self.assertEqual(len(list(self.list)), 333)
            self.assertEqual(len(list(self.list)), 333)

    def testIntIndexingInFirstPage(self):
        with self.replayData("PaginatedList.testGetFirstPage.txt"):
            self.assertEqual(self.list[0].id, 4772349)
            self.assertEqual(self.list[24].id, 4286936)

    def testReversedIterationWithSinglePage(self):
        r = self.list.reversed
        self.assertEqual(r[0].id, 4286936)
        self.assertEqual(r[1].id, 4317009)

    def testReversedIterationWithMultiplePages(self):
        r = self.list.reversed
        self.assertEqual(r[0].id, 94898)
        self.assertEqual(r[1].id, 104702)
        self.assertEqual(r[13].id, 166211)
        self.assertEqual(r[14].id, 166212)
        self.assertEqual(r[15].id, 166214)

    def testReversedIterationSupportsIterator(self):
        # reuse identical test data of testReversedIterationWithSinglePage
        with self.replayData("PaginatedList.testReversedIterationWithSinglePage.txt"):
            r = self.list.reversed
            for i in r:
                self.assertEqual(i.id, 4286936)
                return
            self.fail("empty iterator")

    def testReversedIterationSupportsBuiltinReversed(self):
        # reuse identical test data of testReversedIterationWithSinglePage
        with self.replayData("PaginatedList.testReversedIterationWithSinglePage.txt"):
            for i in reversed(self.list):
                self.assertEqual(i.id, 4286936)
                return
            self.fail("empty iterator")

    def testGettingTheReversedListDoesNotModifyTheOriginalList(self):
        self.assertEqual(self.list[0].id, 18345408)
        self.assertEqual(self.list[30].id, 17916118)
        r = self.list.reversed
        self.assertEqual(self.list[0].id, 18345408)
        self.assertEqual(self.list[30].id, 17916118)
        self.assertEqual(r[0].id, 132373)
        self.assertEqual(r[30].id, 543694)

    def testIntIndexingInThirdPage(self):
        self.assertEqual(self.list[50].id, 3911629)
        self.assertEqual(self.list[74].id, 3605277)

    def testGetFirstPage(self):
        self.assertListKeyEqual(
            self.list.get_page(0),
            lambda i: i.id,
            [
                4772349,
                4767675,
                4758608,
                4700182,
                4662873,
                4608132,
                4604661,
                4588997,
                4557803,
                4554058,
                4539985,
                4507572,
                4507492,
                4507416,
                4447561,
                4406584,
                4384548,
                4383465,
                4373361,
                4373201,
                4370619,
                4356530,
                4352401,
                4317009,
                4286936,
            ],
        )

    def testGetThirdPage(self):
        self.assertListKeyEqual(
            self.list.get_page(2),
            lambda i: i.id,
            [
                3911629,
                3911537,
                3910580,
                3910555,
                3910549,
                3897090,
                3883598,
                3856005,
                3850655,
                3825582,
                3813852,
                3812318,
                3812275,
                3807459,
                3799872,
                3799653,
                3795495,
                3754055,
                3710293,
                3662214,
                3647640,
                3631618,
                3627067,
                3614231,
                3605277,
            ],
        )

    def testIntIndexingAfterIteration(self):
        with self.replayData("PaginatedList.testIteration.txt"):
            self.assertEqual(len(list(self.list)), 333)
            self.assertEqual(self.list[11].id, 4507572)
            self.assertEqual(self.list[73].id, 3614231)
            self.assertEqual(self.list[332].id, 94898)

    def testSliceIndexingInFirstPage(self):
        with self.replayData("PaginatedList.testGetFirstPage.txt"):
            self.assertListKeyEqual(
                self.list[:13],
                lambda i: i.id,
                [
                    4772349,
                    4767675,
                    4758608,
                    4700182,
                    4662873,
                    4608132,
                    4604661,
                    4588997,
                    4557803,
                    4554058,
                    4539985,
                    4507572,
                    4507492,
                ],
            )
            self.assertListKeyEqual(
                self.list[:13:3],
                lambda i: i.id,
                [4772349, 4700182, 4604661, 4554058, 4507492],
            )
            self.assertListKeyEqual(self.list[10:13], lambda i: i.id, [4539985, 4507572, 4507492])
            self.assertListKeyEqual(self.list[5:13:3], lambda i: i.id, [4608132, 4557803, 4507572])

    def testSliceIndexingUntilFourthPage(self):
        self.assertListKeyEqual(
            self.list[:99:10],
            lambda i: i.id,
            [
                4772349,
                4539985,
                4370619,
                4207350,
                4063366,
                3911629,
                3813852,
                3647640,
                3528378,
                3438233,
            ],
        )
        self.assertListKeyEqual(
            self.list[73:78],
            lambda i: i.id,
            [3614231, 3605277, 3596240, 3594731, 3593619],
        )
        self.assertListKeyEqual(
            self.list[70:80:2],
            lambda i: i.id,
            [3647640, 3627067, 3605277, 3594731, 3593430],
        )

    def testSliceIndexingUntilEnd(self):
        with self.replayData("PaginatedList.testIteration.txt"):
            self.assertListKeyEqual(
                self.list[310::3],
                lambda i: i.id,
                [268332, 204247, 169176, 166211, 165898, 163959, 132373, 104702],
            )
            self.assertListKeyEqual(
                self.list[310:],
                lambda i: i.id,
                [
                    268332,
                    211418,
                    205935,
                    204247,
                    172424,
                    171615,
                    169176,
                    166214,
                    166212,
                    166211,
                    166209,
                    166208,
                    165898,
                    165537,
                    165409,
                    163959,
                    132671,
                    132377,
                    132373,
                    130269,
                    111018,
                    104702,
                    94898,
                ],
            )

    def testInterruptedIteration(self):
        # No asserts, but checks that only three pages are fetched
        count = 0
        for element in self.list:  # pragma no branch (exits only by break)
            count += 1
            if count == 75:
                break

    def testInterruptedIterationInSlice(self):
        # reuse identical test data of testInterruptedIteration
        with self.replayData("PaginatedList.testInterruptedIteration.txt"):
            # No asserts, but checks that only three pages are fetched
            count = 0
            # pragma no branch (exits only by break)
            for element in self.list[:100]:
                count += 1
                if count == 75:
                    break

    def testTotalCountWithNoLastPage(self):
        # Fudged replay data, we don't need the data, only the headers
        repos = self.g.get_repos()
        self.assertEqual(0, repos.totalCount)

    def testTotalCountWithDictionary(self):
        # PullRequest.get_review_requests() actually returns a dictionary that
        # we fudge into two lists, which means data is a dict, not a list.
        # We should check the member, not data itself for totalCount.
        pr = self.g.get_repo("PyGithub/PyGithub").get_pull(2078)
        review_requests = pr.get_review_requests()
        self.assertEqual(review_requests[0].totalCount, 0)
        self.assertEqual(review_requests[1].totalCount, 0)

    def testTotalCountWithDeprecationLink(self):
        # Test the original reported scenario: search_issues with commit SHA
        issues = self.g.search_issues("commit:example_sha")
        # Should return the actual count from JSON, not 0
        self.assertEqual(issues.totalCount, 1)

    def testCustomPerPage(self):
        self.assertEqual(self.g.per_page, 30)
        self.g.per_page = 100
        self.assertEqual(self.g.per_page, 100)
        self.assertEqual(len(list(self.repo.get_issues())), 456)

    def testCustomPerPageWithNoUrlParams(self):
        self.g.per_page = 100
        self.assertEqual(len(list(self.repo.get_comments())), 325)

    def testCustomPerPageWithGetPage(self):
        self.g.per_page = 100
        self.assertEqual(len(self.repo.get_issues().get_page(2)), 100)

    def testCustomPerPageIteration(self):
        self.g.per_page = 3
        repo = self.g.get_repo("PyGithub/PyGithub")
        comments = repo.get_issue(1136).get_comments()
        self.assertEqual(
            [
                datetime(2019, 8, 10, 18, 16, 46, tzinfo=timezone.utc),
                datetime(2024, 1, 6, 16, 4, 34, tzinfo=timezone.utc),
                datetime(2024, 1, 6, 17, 34, 11, tzinfo=timezone.utc),
                datetime(2024, 3, 20, 15, 24, 15, tzinfo=timezone.utc),
                datetime(2024, 3, 21, 10, 55, 14, tzinfo=timezone.utc),
                datetime(2024, 3, 21, 14, 2, 22, tzinfo=timezone.utc),
                datetime(2024, 3, 24, 13, 58, 57, tzinfo=timezone.utc),
            ],
            [comment.created_at for comment in comments],
        )

    def testCustomPerPageReversedIteration(self):
        self.g.per_page = 3
        repo = self.g.get_repo("PyGithub/PyGithub")
        comments = repo.get_issue(1136).get_comments().reversed
        self.assertEqual(
            [
                datetime(2024, 3, 24, 13, 58, 57, tzinfo=timezone.utc),
                datetime(2024, 3, 21, 14, 2, 22, tzinfo=timezone.utc),
                datetime(2024, 3, 21, 10, 55, 14, tzinfo=timezone.utc),
                datetime(2024, 3, 20, 15, 24, 15, tzinfo=timezone.utc),
                datetime(2024, 1, 6, 17, 34, 11, tzinfo=timezone.utc),
                datetime(2024, 1, 6, 16, 4, 34, tzinfo=timezone.utc),
                datetime(2019, 8, 10, 18, 16, 46, tzinfo=timezone.utc),
            ],
            [comment.created_at for comment in comments],
        )

    def testCustomPerPageReversedIterationSinglePage(self):
        self.g.per_page = 4
        repo = self.g.get_repo("PyGithub/PyGithub")
        comments = repo.get_issue(3372).get_comments().reversed
        self.assertEqual(
            [
                datetime(2025, 9, 17, 19, 49, 40, tzinfo=timezone.utc),
                datetime(2025, 9, 17, 8, 44, 18, tzinfo=timezone.utc),
                datetime(2025, 9, 16, 19, 22, 2, tzinfo=timezone.utc),
                datetime(2025, 9, 15, 12, 15, 13, tzinfo=timezone.utc),
            ],
            [comment.created_at for comment in comments],
        )

    # more custom per page tests in CompletableGithubObjectWithPaginatedProperty

    def testWithFirstPage(self):
        # fetching the commit also fetches the fist page of files
        with self.captureRequests() as requests:
            repo = self.g.get_repo("PyGithub/PyGithub", lazy=True)
            commit = repo.get_commit("e359b83a04e8f34bedab0f2180169012d238a135", commit_files_per_page=3)
            # repo is lazy, so this commit is also lazy, here we test with an eager (fetched) commit
            commit.complete()
            files = commit.files
        self.assertListKeyEqual(
            requests,
            lambda r: r.url,
            ["/repos/PyGithub/PyGithub/commits/e359b83a04e8f34bedab0f2180169012d238a135?page=1&per_page=3"],
        )

        # consuming the first page of files should not fire a request
        with self.captureRequests() as requests:
            self.assertEqual(files[0].filename, "github/GeneratedReleaseNotes.py")
            self.assertEqual(files[1].filename, "github/Repository.py")
            self.assertEqual(files[2].filename, "pyproject.toml")
        self.assertEqual(len(requests), 0)

        # consuming items of the second page fetches the second page
        with self.captureRequests() as requests:
            self.assertEqual(files[3].filename, "tests/ReplayData/Repository.testGenerateReleaseNotes.txt")
        self.assertListKeyEqual(
            requests,
            lambda r: r.url,
            ["/repositories/3544490/commits/e359b83a04e8f34bedab0f2180169012d238a135?page=2&per_page=3"],
        )

        # consuming further items of the second page does not fire a request
        with self.captureRequests() as requests:
            self.assertEqual(
                files[4].filename, "tests/ReplayData/Repository.testGenerateReleaseNotesWithAllArguments.txt"
            )
            self.assertEqual(files[5].filename, "tests/Repository.py")
        self.assertEqual(len(requests), 0)

        # consuming items of the last page fetches the that page
        with self.captureRequests() as requests:
            self.assertEqual(files[6].filename, "tests/test_release_notes.yml")
        self.assertListKeyEqual(
            requests,
            lambda r: r.url,
            ["/repositories/3544490/commits/e359b83a04e8f34bedab0f2180169012d238a135?page=3&per_page=3"],
        )

    def testWithFirstSinglePage(self):
        # fetching the commit also fetches the fist page of files
        with self.captureRequests() as requests:
            repo = self.g.get_repo("PyGithub/PyGithub", lazy=True)
            commit = repo.get_commit("f5f9756a1dd52a53820cc54927abb34725377987", commit_files_per_page=3)
            # repo is lazy, so this commit is also lazy, here we test with an eager (fetched) commit
            commit.complete()
            files = commit.files
            # with a single page we by now know the total count without firing a request
            self.assertEqual(files.totalCount, 3)
        self.assertListKeyEqual(
            requests,
            lambda r: r.url,
            ["/repos/PyGithub/PyGithub/commits/f5f9756a1dd52a53820cc54927abb34725377987?page=1&per_page=3"],
        )

        # consuming the first page of files should not fire a request
        with self.captureRequests() as requests:
            self.assertEqual(files[0].filename, "github/GithubApp.py")
            self.assertEqual(files[1].filename, "github/GithubObject.py")
            self.assertEqual(files[2].filename, "tests/GithubObject.py")
        self.assertEqual(len(requests), 0)

    def testReversedWithFirstPage(self):
        # this is all lazy, no requests fired
        with self.captureRequests() as requests:
            repo = self.g.get_repo("PyGithub/PyGithub", lazy=True)
            commit = repo.get_commit("e359b83a04e8f34bedab0f2180169012d238a135", commit_files_per_page=3)
            # repo is lazy, so this commit is also lazy, here we test with an eager (fetched) commit
            commit.complete()
            files = commit.files
        self.assertListKeyEqual(
            requests,
            lambda r: r.url,
            ["/repos/PyGithub/PyGithub/commits/e359b83a04e8f34bedab0f2180169012d238a135?page=1&per_page=3"],
        )

        # reversing files does not fire a request because there is a last Link header in the first response
        with self.captureRequests() as requests:
            files = files.reversed
        self.assertEqual(len(requests), 0)

        # consuming the first page of the reversed files fetches the "last" URL
        with self.captureRequests() as requests:
            self.assertEqual(files[0].filename, "tests/test_release_notes.yml")
        self.assertListKeyEqual(
            requests,
            lambda r: r.url,
            ["/repositories/3544490/commits/e359b83a04e8f34bedab0f2180169012d238a135?page=3&per_page=3"],
        )

        # consuming items of the second page fetches the second page
        with self.captureRequests() as requests:
            self.assertEqual(files[1].filename, "tests/Repository.py")
        self.assertListKeyEqual(
            requests,
            lambda r: r.url,
            ["/repositories/3544490/commits/e359b83a04e8f34bedab0f2180169012d238a135?page=2&per_page=3"],
        )
        # consuming further items of the second page does not fire a request
        with self.captureRequests() as requests:
            self.assertEqual(
                files[2].filename, "tests/ReplayData/Repository.testGenerateReleaseNotesWithAllArguments.txt"
            )
            self.assertEqual(files[3].filename, "tests/ReplayData/Repository.testGenerateReleaseNotes.txt")
        self.assertEqual(len(requests), 0)

        # consuming items of the last page fetches the that page
        with self.captureRequests() as requests:
            self.assertEqual(files[4].filename, "pyproject.toml")
            self.assertEqual(files[5].filename, "github/Repository.py")
            self.assertEqual(files[6].filename, "github/GeneratedReleaseNotes.py")
        self.assertListKeyEqual(
            requests,
            lambda r: r.url,
            ["/repositories/3544490/commits/e359b83a04e8f34bedab0f2180169012d238a135?page=1&per_page=3"],
        )

    def testReversedWithFirstSinglePage(self):
        # fetching the commit also fetches the fist page of files
        with self.captureRequests() as requests:
            repo = self.g.get_repo("PyGithub/PyGithub", lazy=True)
            commit = repo.get_commit("f5f9756a1dd52a53820cc54927abb34725377987", commit_files_per_page=3)
            # repo is lazy, so this commit is also lazy, here we test with an eager (fetched) commit
            commit.complete()
            files = commit.files
            # with a single page we by now know the total count without firing a request
            self.assertEqual(files.totalCount, 3)
        self.assertListKeyEqual(
            requests,
            lambda r: r.url,
            [
                "/repos/PyGithub/PyGithub/commits/f5f9756a1dd52a53820cc54927abb34725377987?page=1&per_page=3",
            ],
        )

        # reversing files fires another request because there were no Link headers in the first response
        with self.captureRequests() as requests:
            files = files.reversed
        self.assertListKeyEqual(
            requests, lambda r: r.url, ["/repos/PyGithub/PyGithub/commits/f5f9756a1dd52a53820cc54927abb34725377987"]
        )

        # consuming the first page of files should not fire a request
        with self.captureRequests() as requests:
            self.assertEqual(files[0].filename, "tests/GithubObject.py")
            self.assertEqual(files[1].filename, "github/GithubObject.py")
            self.assertEqual(files[2].filename, "github/GithubApp.py")
        self.assertEqual(len(requests), 0)

    def testNoFirstPage(self):
        self.assertFalse(next(iter(self.list), None))

    def testMergeDicts(self):
        self.assertDictEqual(
            PaginatedListImpl.merge_dicts(
                {"a": 1, "b": 2, "c": 3},
                {"c": 4, "d": 5, "e": 6},
            ),
            {"a": 1, "b": 2, "c": 4, "d": 5, "e": 6},
        )

    def testOverrideAttributes(self):
        input_dict = {"a": 1, "b": 2, "c": 3}
        overrides_dict = {"c": 4, "d": 5, "e": 6}
        transformer = PaginatedListImpl.override_attributes(overrides_dict)
        self.assertDictEqual(transformer(input_dict), {"a": 1, "b": 2, "c": 4, "d": 5, "e": 6})

    def testGraphQlPagination(self):
        repo = self.g.get_repo("PyGithub/PyGithub")
        discussions = repo.get_discussions("id number")
        self.assertFalse(discussions.is_rest)
        self.assertTrue(discussions.is_graphql)
        rev = discussions.reversed

        discussions_list = list(discussions)
        self.assertEqual(discussions.totalCount, 65)
        self.assertEqual(len(discussions_list), 65)
        self.assertEqual(discussions_list[0].number, 3044)
        self.assertEqual(discussions_list[-1].number, 1780)

        reversed_list = list(rev)
        self.assertEqual(rev.totalCount, 65)
        self.assertEqual(len(reversed_list), 65)
        self.assertListEqual([d.number for d in reversed_list], [d.number for d in reversed(discussions_list)])

        # accessing totalCount before iterating the PaginatedList triggers another request
        self.assertEqual(repo.get_discussions("id number").totalCount, 65)



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2021 Claire Johns <42869556+johnsc1@users.noreply.github.com>      #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Tim Gates <tim.gates@iress.com>                               #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from . import Framework


class GitCommitVerification(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.commit = (
            self.g.get_repo("PyGithub/PyGithub", lazy=True)
            .get_commit("801d64a4c5c0fcb63f695e0f6799117e76e5fe67")
            .complete()
        )

    def testAttributes(self):
        verification = self.commit.commit.verification
        self.assertEqual(
            verification.payload,
            "tree 4f502b4c4e5f0bdc7ee611f914bbdef51aef5efa\nparent 85087354078e426125dbbf88041bbaa6f35d8199\nauthor Tim Gates <tim.gates@iress.com> 1724830907 +1000\ncommitter Tim Gates <tim.gates@iress.com> 1724830915 +1000\n\nCommit verification support\n\nAdd support for verification component of Commit API response to see if\ncommit has been signed and the signature has been checked by Github\n",
        )
        self.assertEqual(verification.reason, "valid")
        self.assertEqual(
            verification.signature,
            "-----BEGIN PGP SIGNATURE-----\n\niMgEAAEKADIWIQRbWfY9TlELdM/5UaauO+DVOCPPBQUCZs7U0RQcdGltLmdhdGVz\nQGlyZXNzLmNvbQAKCRCuO+DVOCPPBQ2tBACAaRNONEWlDDNyYkAnIv8bZ55BuIuy\nTvbxVPjI8KLDqKLzgHO60HhQ3h/hCiug6g5fvVzyIrmayj3eEzaWAfa3+f37f3xK\nflZRMtNFUBYQoLTuyTkKvW85UA2AkUvKp3bHT5W6ZoCKqR5xw6pwKxuYQi7eJG9h\nNPrZTezkL6EDng==\n=Ng9G\n-----END PGP SIGNATURE-----",
        )
        self.assertEqual(verification.verified, True)
        self.assertIsNone(verification.verified_at)
        self.assertEqual(verification.reason, "valid")
        self.assertRegex(verification.signature, ".*PGP.*")
        self.assertRegex(verification.payload, ".*tree.*")



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 netsgnut <284779+netsgnut@users.noreply.github.com>           #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from . import Framework


class Issue945(Framework.TestCase):  # https://github.com/PyGithub/PyGithub/issues/945
    def setUp(self):
        super().setUp()
        self.repo = self.g.get_user("openframeworks").get_repo("openFrameworks")
        self.list = self.repo.get_issues()
        self.list_with_headers = self.repo.get_stargazers_with_dates()

    def testReservedPaginatedListAttributePreservation(self):
        r1 = self.list.reversed
        self.assertEqual(self.list._PaginatedList__contentClass, r1._PaginatedList__contentClass)
        self.assertEqual(self.list._PaginatedList__requester, r1._PaginatedList__requester)
        self.assertEqual(self.list._PaginatedList__firstUrl, r1._PaginatedList__firstUrl)
        self.assertEqual(self.list._PaginatedList__firstParams, r1._PaginatedList__firstParams)
        self.assertEqual(self.list._PaginatedList__headers, r1._PaginatedList__headers)
        self.assertEqual(self.list._PaginatedList__list_item, r1._PaginatedList__list_item)

        self.assertTrue(self.list_with_headers._PaginatedList__headers is not None)
        r2 = self.list_with_headers.reversed
        self.assertEqual(
            self.list_with_headers._PaginatedList__contentClass,
            r2._PaginatedList__contentClass,
        )
        self.assertEqual(
            self.list_with_headers._PaginatedList__requester,
            r2._PaginatedList__requester,
        )
        self.assertEqual(self.list_with_headers._PaginatedList__firstUrl, r2._PaginatedList__firstUrl)
        self.assertEqual(
            self.list_with_headers._PaginatedList__firstParams,
            r2._PaginatedList__firstParams,
        )
        self.assertEqual(self.list_with_headers._PaginatedList__headers, r2._PaginatedList__headers)
        self.assertEqual(
            self.list_with_headers._PaginatedList__list_item,
            r2._PaginatedList__list_item,
        )



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2021 Claire Johns <42869556+johnsc1@users.noreply.github.com>      #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from . import Framework


class PullRequestReview1856(Framework.TestCase):
    def testDelete(self):
        pumpkin_repo = self.g.get_repo("CS481-Team-Pumpkin/PyGithub", lazy=True)
        pumpkin_pull = pumpkin_repo.get_pull(4)
        pullreview = pumpkin_pull.get_review(631460061)

        pullreview.delete()
        reviews = pumpkin_pull.get_reviews()
        self.assertEqual(list(reviews), [])



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2021 karthik-kadajji <60779081+karthik-kadajji@users.noreply.github.com>#
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from . import Framework


class Organization(Framework.TestCase):
    def testIssue2030CreateProject(self):
        super().setUp()
        project = "ultratendency"
        self.user = self.g.get_user("karthik-kadajji-t")
        self.org = self.g.get_organization("testkarthik")
        self.org.create_project(project)
        project = self.org.get_projects()[0].name

        self.assertEqual("ultratendency", project)
